@@ -17,7 +17,7 @@
 %                               February 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -154,7 +154,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
       for ( ; ; )
       {
         text=AcquireString(caption);
-        i=FormatMagickCaption(image,draw_info,text,&metrics);
+        i=FormatMagickCaption(image,draw_info,&metrics,&text);
         (void) CloneString(&draw_info->text,text);
         text=DestroyString(text);
         (void) FormatMagickString(geometry,MaxTextExtent,"%+g%+g",
@@ -162,8 +162,9 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
         if (draw_info->gravity == UndefinedGravity)
           (void) CloneString(&draw_info->geometry,geometry);
         status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-        width=(unsigned long) (metrics.width+draw_info->stroke_width+0.5);
-        height=(unsigned long) (metrics.height+draw_info->stroke_width+0.5);
+        width=(unsigned long) floor(metrics.width+draw_info->stroke_width+0.5);
+        height=(unsigned long) floor(metrics.height+draw_info->stroke_width+
+          0.5);
         if ((width > (image->columns+1)) || (height > (image->rows+1)))
           break;
         draw_info->pointsize*=2.0;
@@ -172,7 +173,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
       for ( ; ; )
       {
         text=AcquireString(caption);
-        i=FormatMagickCaption(image,draw_info,text,&metrics);
+        i=FormatMagickCaption(image,draw_info,&metrics,&text);
         (void) CloneString(&draw_info->text,text);
         text=DestroyString(text);
         (void) FormatMagickString(geometry,MaxTextExtent,"%+g%+g",
@@ -180,32 +181,42 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
         if (draw_info->gravity == UndefinedGravity)
           (void) CloneString(&draw_info->geometry,geometry);
         status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-        width=(unsigned long) (metrics.width+draw_info->stroke_width+0.5);
-        height=(unsigned long) (metrics.height+draw_info->stroke_width+0.5);
+        width=(unsigned long) floor(metrics.width+draw_info->stroke_width+0.5);
+        height=(unsigned long) floor(metrics.height+draw_info->stroke_width+
+          0.5);
         if ((width > (image->columns+1)) || (height > (image->rows+1)))
           break;
         draw_info->pointsize++;
       }
       draw_info->pointsize--;
     }
-  i=FormatMagickCaption(image,draw_info,caption,&metrics);
+  i=FormatMagickCaption(image,draw_info,&metrics,&caption);
   if (image->rows == 0)
     image->rows=(unsigned long) ((i+1)*(metrics.ascent-metrics.descent+
       draw_info->stroke_width)+0.5);
   if (image->rows == 0)
     image->rows=(unsigned long) ((i+1)*draw_info->pointsize+
       draw_info->stroke_width+0.5);
-  (void) SetImageBackgroundColor(image);
+  if (SetImageBackgroundColor(image) == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Draw caption.
   */
   (void) CloneString(&draw_info->text,caption);
   status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-  (void) FormatMagickString(geometry,MaxTextExtent,"%+g%+g",
-    -metrics.bounds.x1+draw_info->stroke_width/2.0,metrics.ascent+
-    draw_info->stroke_width/2.0);
-  if (draw_info->gravity == UndefinedGravity)
-    (void) CloneString(&draw_info->geometry,geometry);
+  if (draw_info->gravity != UndefinedGravity)
+    image->page.x=(long) (metrics.bounds.x1-draw_info->stroke_width/2.0);
+  else
+    {
+      (void) FormatMagickString(geometry,MaxTextExtent,"%+g%+g",
+        -metrics.bounds.x1+draw_info->stroke_width/2.0,metrics.ascent+
+        draw_info->stroke_width/2.0);
+      (void) CloneString(&draw_info->geometry,geometry);
+    }
   (void) AnnotateImage(image,draw_info);
   draw_info=DestroyDrawInfo(draw_info);
   caption=DestroyString(caption);
