@@ -16,7 +16,7 @@
 %                              Bill Radcliffe                                 %
 %                                   2001                                      %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -83,8 +83,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -118,8 +117,7 @@ static MagickBooleanType IsEMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -290,8 +288,7 @@ static wchar_t *ConvertUTF8ToUTF16(const unsigned char *source)
 static HENHMETAFILE ReadEnhMetaFile(const char *path,long *width,
   long *height)
 {
-#pragma pack( push )
-#pragma pack( 2 )
+#pragma pack( push, 2 )
   typedef struct
   {
     DWORD dwKey;
@@ -529,10 +526,11 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
           if (image->x_resolution != 0.0)
-            image->columns=(unsigned long) ((image->columns*
+            image->columns=(unsigned long) floor((image->columns*
               image->x_resolution)+0.5);
           if (image->y_resolution != 0.0)
-            image->rows=(unsigned long) ((image->rows*image->y_resolution)+0.5);
+            image->rows=(unsigned long) floor((image->rows*image->y_resolution)+
+              0.5);
         }
       else
         {
@@ -540,11 +538,11 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
           if (image->x_resolution != 0.0)
-            image->columns=(unsigned long) (((image->columns*
+            image->columns=(unsigned long) floor(((image->columns*
               image->x_resolution)/DefaultResolution)+0.5);
           if (image->y_resolution != 0.0)
-            image->rows=(unsigned long) (((image->rows*image->y_resolution)/
-              DefaultResolution)+0.5);
+            image->rows=(unsigned long) floor(((image->rows*
+              image->y_resolution)/DefaultResolution)+0.5);
         }
       geometry=DestroyString(geometry);
     }
@@ -557,11 +555,6 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   /*
     Initialize the bitmap header info.
   */
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   (void) ResetMagickMemory(&DIBinfo,0,sizeof(BITMAPINFO));
   DIBinfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
   DIBinfo.bmiHeader.biWidth=image->columns;
@@ -625,7 +618,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
       q->red=ScaleCharToQuantum(pBits->rgbRed);
       q->green=ScaleCharToQuantum(pBits->rgbGreen);
       q->blue=ScaleCharToQuantum(pBits->rgbBlue);
-      q->opacity=OpaqueOpacity;
+      SetOpacityPixelComponent(q,OpaqueOpacity);
       pBits++;
       q++;
     }
