@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -80,17 +80,17 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadLABELImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
   char
-    geometry[MaxTextExtent];
+    geometry[MaxTextExtent],
+    *property;
 
   const char
     *label;
@@ -121,14 +121,16 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   (void) ResetImagePage(image,"0x0+0+0");
-  (void) SetImageProperty(image,"label",InterpretImageProperties(image_info,
-    image,image_info->filename));
+  property=InterpretImageProperties(image_info,image,image_info->filename);
+  (void) SetImageProperty(image,"label",property);
+  property=DestroyString(property);
   label=GetImageProperty(image,"label");
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   draw_info->text=ConstantString(label);
-  if ((image->columns != 0) || (image->rows != 0))
+  if (((image->columns != 0) || (image->rows != 0)) &&
+      (image_info->pointsize == 0.0))
     {
       /*
         Fit label to canvas size.
@@ -136,8 +138,9 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       status=GetMultilineTypeMetrics(image,draw_info,&metrics);
       for ( ; status != MagickFalse; draw_info->pointsize*=2.0)
       {
-        width=(unsigned long) (metrics.width+draw_info->stroke_width+0.5);
-        height=(unsigned long) (metrics.height+draw_info->stroke_width+0.5);
+        width=(unsigned long) floor(metrics.width+draw_info->stroke_width+0.5);
+        height=(unsigned long) floor(metrics.height+draw_info->stroke_width+
+          0.5);
         if (((image->columns != 0) && (width > (image->columns+1))) ||
             ((image->rows != 0) && (height > (image->rows+1))))
           break;
@@ -145,8 +148,9 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       }
       for ( ; status != MagickFalse; draw_info->pointsize--)
       {
-        width=(unsigned long) (metrics.width+draw_info->stroke_width+0.5);
-        height=(unsigned long) (metrics.height+draw_info->stroke_width+0.5);
+        width=(unsigned long) floor(metrics.width+draw_info->stroke_width+0.5);
+        height=(unsigned long) floor(metrics.height+draw_info->stroke_width+
+          0.5);
         if ((image->columns != 0) && (width <= (image->columns+1)) &&
            ((image->rows == 0) || (height <= (image->rows+1))))
           break;
@@ -166,10 +170,10 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       return((Image *) NULL);
     }
   if (image->columns == 0)
-    image->columns=(unsigned long) (metrics.width+draw_info->stroke_width+0.5);
+    image->columns=(unsigned long) (metrics.width+draw_info->stroke_width+1.5);
   if (image->columns == 0)
     image->columns=(unsigned long) (draw_info->pointsize+
-      draw_info->stroke_width+0.5);
+      draw_info->stroke_width+1.5);
   if (draw_info->gravity == UndefinedGravity)
     {
       (void) FormatMagickString(geometry,MaxTextExtent,"%+g%+g",
@@ -178,16 +182,17 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       draw_info->geometry=AcquireString(geometry);
     }
   if (image->rows == 0)
-    image->rows=(unsigned long) (metrics.height+draw_info->stroke_width+0.5);
-  if (image->rows == 0)
-    image->rows=(unsigned long) (draw_info->pointsize+draw_info->stroke_width+
+    image->rows=(unsigned long) floor(metrics.height+draw_info->stroke_width+
       0.5);
-  if (SetImageExtent(image,0,0) == MagickFalse)
+  if (image->rows == 0)
+    image->rows=(unsigned long) floor(draw_info->pointsize+
+      draw_info->stroke_width+0.5);
+  if (SetImageBackgroundColor(image) == MagickFalse)
     {
       InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
+      image=DestroyImageList(image);
+      return((Image *) NULL);
     }
-  (void) SetImageBackgroundColor(image);
   (void) AnnotateImage(image,draw_info);
   draw_info=DestroyDrawInfo(draw_info);
   return(GetFirstImageInList(image));
@@ -224,6 +229,7 @@ ModuleExport unsigned long RegisterLABELImage(void)
   entry=SetMagickInfo("LABEL");
   entry->decoder=(DecodeImageHandler *) ReadLABELImage;
   entry->adjoin=MagickFalse;
+  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Image label");
   entry->module=ConstantString("LABEL");
   (void) RegisterMagickInfo(entry);
