@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -73,6 +73,10 @@
 #include "magick/transform.h"
 #include "magick/utility.h"
 #include "magick/module.h"
+
+/*
+  Define declarations.
+*/
 #if defined(MAGICKCORE_TIFF_DELEGATE)
 #define CCITTParam  "-1"
 #else
@@ -90,19 +94,18 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I n v o k e P o s t s r i p t D e l e g a t e                             %
+%   I n v o k e P D F D e l e g a t e                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  InvokePostscriptDelegate() executes the postscript interpreter with the
-%  specified command.
+%  InvokePDFDelegate() executes the PDF interpreter with the specified command.
 %
-%  The format of the InvokePostscriptDelegate method is:
+%  The format of the InvokePDFDelegate method is:
 %
-%      MagickBooleanType InvokePostscriptDelegate(
-%        const MagickBooleanType verbose,const char *command)
+%      MagickBooleanType InvokePDFDelegate(const MagickBooleanType verbose,
+%        const char *command,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -112,76 +115,96 @@ static MagickBooleanType
 %    o command: the address of a character string containing the command to
 %      execute.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-static MagickBooleanType InvokePostscriptDelegate(
-  const MagickBooleanType verbose,const char *command)
+static MagickBooleanType InvokePDFDelegate(const MagickBooleanType verbose,
+  const char *command,ExceptionInfo *exception)
 {
+  int
+    status;
+
 #if defined(MAGICKCORE_GS_DELEGATE) || defined(__WINDOWS__)
   char
     **argv;
 
-  const GhostscriptVectors
-    *gs_func;
+  const GhostInfo
+    *ghost_info;
 
   gs_main_instance
     *interpreter;
 
   int
     argc,
-    code,
-    status;
+    code;
 
   register long
     i;
 
 #if defined(__WINDOWS__)
-  gs_func=NTGhostscriptDLLVectors();
+  ghost_info=NTGhostscriptDLLVectors();
 #else
-  GhostscriptVectors
-    gs_func_struct;
+  GhostInfo
+    ghost_info_struct;
 
-  gs_func=(&gs_func_struct);
-  (void) ResetMagickMemory(&gs_func,0,sizeof(gs_func));
-  gs_func_struct.new_instance=(int (*)(gs_main_instance **,void *))
+  ghost_info=(&ghost_info_struct);
+  (void) ResetMagickMemory(&ghost_info,0,sizeof(ghost_info));
+  ghost_info_struct.new_instance=(int (*)(gs_main_instance **,void *))
     gsapi_new_instance;
-  gs_func_struct.init_with_args=(int (*)(gs_main_instance *,int,char **))
+  ghost_info_struct.init_with_args=(int (*)(gs_main_instance *,int,char **))
     gsapi_init_with_args;
-  gs_func_struct.run_string=(int (*)(gs_main_instance *,const char *,int,int *))
-    gsapi_run_string;
-  gs_func_struct.delete_instance=(void (*)(gs_main_instance *))
+  ghost_info_struct.run_string=(int (*)(gs_main_instance *,const char *,int,
+    int *)) gsapi_run_string;
+  ghost_info_struct.delete_instance=(void (*)(gs_main_instance *))
     gsapi_delete_instance;
-  gs_func_struct.exit=(int (*)(gs_main_instance *)) gsapi_exit;
+  ghost_info_struct.exit=(int (*)(gs_main_instance *)) gsapi_exit;
 #endif
-  if (gs_func == (GhostscriptVectors *) NULL)
-    return(SystemCommand(verbose,command) == 0 ? MagickFalse : MagickTrue);
+  if (ghost_info == (GhostInfo *) NULL)
+    {
+      status=SystemCommand(MagickFalse,verbose,command,exception);
+      return(status == 0 ? MagickTrue : MagickFalse);
+    }
   if (verbose != MagickFalse)
     {
       (void) fputs("[ghostscript library]",stdout);
       (void) fputs(strchr(command,' '),stdout);
     }
-  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
+  status=(ghost_info->new_instance)(&interpreter,(void *) NULL);
   if (status < 0)
-    return(SystemCommand(verbose,command) == 0 ? MagickFalse : MagickTrue);
+    {
+      status=SystemCommand(MagickFalse,verbose,command,exception);
+      return(status == 0 ? MagickTrue : MagickFalse);
+    }
   argv=StringToArgv(command,&argc);
-  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
+  status=(ghost_info->init_with_args)(interpreter,argc-1,argv+1);
   if (status == 0)
-    status=(gs_func->run_string)(interpreter,"systemdict /start get exec\n",0,
-      &code);
-  (gs_func->exit)(interpreter);
-  (gs_func->delete_instance)(interpreter);
+    status=(ghost_info->run_string)(interpreter,"systemdict /start get exec\n",
+      0,&code);
+  (ghost_info->exit)(interpreter);
+  (ghost_info->delete_instance)(interpreter);
 #if defined(__WINDOWS__)
   NTGhostscriptUnLoadDLL();
 #endif
   for (i=0; i < (long) argc; i++)
     argv[i]=DestroyString(argv[i]);
   argv=(char **) RelinquishMagickMemory(argv);
-  if ((status == 0) || (status == -101))
-    return(MagickFalse);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-    "Ghostscript returns status %d, exit code %d",status,code);
+  if ((status != 0) && (status != -101))
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      (void) ThrowMagickException(exception,GetMagickModule(),DelegateError,
+        "`%s': %s",command,message);
+      message=DestroyString(message);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Ghostscript returns status %d, exit code %d",status,code);
+      return(MagickFalse);
+    }
   return(MagickTrue);
 #else
-  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
+  status=SystemCommand(MagickFalse,verbose,command,exception);
+  return(status == 0 ? MagickTrue : MagickFalse);
 #endif
 }
 
@@ -205,8 +228,7 @@ static MagickBooleanType InvokePostscriptDelegate(
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o offset: Specifies the offset of the magick string.
 %
@@ -215,7 +237,7 @@ static MagickBooleanType IsPDF(const unsigned char *magick,const size_t offset)
 {
   if (offset < 5)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick,"%PDF-",5) == 0)
+  if (LocaleNCompare((const char *) magick,"%PDF-",5) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -268,6 +290,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define CropBox  "CropBox"
 #define DeviceCMYK  "DeviceCMYK"
+#define ICCBased  "ICCBased"
 #define MediaBox  "MediaBox"
 #define RenderPostscriptText  "Rendering Postscript...  "
 #define PDFRotate  "Rotate"
@@ -295,7 +318,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   Image
     *image,
-    *next_image,
+    *next,
     *pdf_image;
 
   ImageInfo
@@ -410,7 +433,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       c=' ';
     *p++=(char) c;
     if ((c != (int) '/') && (c != (int) '%') && 
-        ((size_t) (p-command) < MaxTextExtent))
+        ((size_t) (p-command) < (MaxTextExtent-1)))
       continue;
     *(--p)='\0';
     p=command;
@@ -421,6 +444,8 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     if (LocaleNCompare(DeviceCMYK,command,strlen(DeviceCMYK)) == 0)
       cmyk=MagickTrue;
+    if (LocaleNCompare(ICCBased,command,strlen(ICCBased)) == 0)
+      cmyk=MagickTrue;
     if (LocaleNCompare(SpotColor,command,strlen(SpotColor)) == 0)
       {
         char
@@ -502,11 +527,12 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Set PDF render geometry.
         */
-        (void) FormatMagickString(geometry,MaxTextExtent,"%gx%g%+g%+g",
-          bounds.x2-bounds.x1,bounds.y2-bounds.y1,bounds.x1,bounds.y1);
+        (void) FormatMagickString(geometry,MaxTextExtent,
+          "%gx%g%+.15g%+.15g",bounds.x2-bounds.x1,bounds.y2-bounds.y1,
+           bounds.x1,bounds.y1);
         (void) SetImageProperty(image,"pdf:HiResBoundingBox",geometry);
-        page.width=(unsigned long) (bounds.x2-bounds.x1+0.5);
-        page.height=(unsigned long) (bounds.y2-bounds.y1+0.5);
+        page.width=(unsigned long) floor(bounds.x2-bounds.x1+0.5);
+        page.height=(unsigned long) floor(bounds.y2-bounds.y1+0.5);
         hires_bounds=bounds;
       }
   }
@@ -533,7 +559,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImage(image);
       return((Image *) NULL);
     }
-  (void) write(file," ",1);
+  count=write(file," ",1);
   file=close(file)-1;
   /*
     Render Postscript with the Ghostscript delegate.
@@ -556,13 +582,15 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   *options='\0';
-  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",image->x_resolution,
-    image->y_resolution);
+  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",
+    image->x_resolution,image->y_resolution);
   if (image_info->page != (char *) NULL)
     {
       (void) ParseAbsoluteGeometry(image_info->page,&page);
-      page.width=(unsigned long) (page.width*image->x_resolution/delta.x+0.5);
-      page.height=(unsigned long) (page.height*image->y_resolution/delta.y+0.5);
+      page.width=(unsigned long) floor(page.width*image->x_resolution/delta.x+
+        0.5);
+      page.height=(unsigned long) floor(page.height*image->y_resolution/delta.y+
+        0.5);
       (void) FormatMagickString(options,MaxTextExtent,"-g%lux%lu ",page.width,
         page.height);
     }
@@ -597,9 +625,9 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,density,options,
     read_info->filename,postscript_filename,input_filename);
-  status=InvokePostscriptDelegate(read_info->verbose,command);
+  status=InvokePDFDelegate(read_info->verbose,command,exception);
   pdf_image=(Image *) NULL;
-  if ((status == MagickFalse) &&
+  if ((status != MagickFalse) &&
       (IsPDFRendered(read_info->filename) != MagickFalse))
     pdf_image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(postscript_filename);
@@ -642,19 +670,23 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           PrependImageToList(&pdf_image,clone_image);
       }
     }
-  scene=0;
   do
   {
     (void) CopyMagickString(pdf_image->filename,filename,MaxTextExtent);
     pdf_image->page=page;
-    pdf_image->scene=scene++;
     (void) CloneImageProfiles(pdf_image,image);
     (void) CloneImageProperties(pdf_image,image);
-    next_image=SyncNextImageInList(pdf_image);
-    if (next_image != (Image *) NULL)
-      pdf_image=next_image;
-  } while (next_image != (Image *) NULL);
+    next=SyncNextImageInList(pdf_image);
+    if (next != (Image *) NULL)
+      pdf_image=next;
+  } while (next != (Image *) NULL);
   image=DestroyImage(image);
+  scene=0;
+  for (next=GetFirstImageInList(pdf_image); next != (Image *) NULL; )
+  {
+    next->scene=scene++;
+    next=GetNextImageInList(next);
+  }
   return(GetFirstImageInList(pdf_image));
 }
 
@@ -825,6 +857,43 @@ static char *EscapeParenthesis(const char *text)
   return(buffer);
 }
 
+static MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
+  Image *image,Image *inject_image)
+{
+  Image
+    *group4_image;
+
+  ImageInfo
+    *write_info;
+
+  MagickBooleanType
+    status;
+
+  size_t
+    length;
+
+  unsigned char
+    *group4;
+
+  status=MagickTrue;
+  write_info=CloneImageInfo(image_info);
+  (void) CopyMagickString(write_info->filename,"GROUP4:",MaxTextExtent);
+  (void) CopyMagickString(write_info->magick,"GROUP4",MaxTextExtent);
+  group4_image=CloneImage(inject_image,0,0,MagickTrue,&image->exception);
+  if (group4_image == (Image *) NULL)
+    return(MagickFalse);
+  group4=(unsigned char *) ImageToBlob(write_info,group4_image,&length,
+    &image->exception);
+  group4_image=DestroyImage(group4_image);
+  if (group4 == (unsigned char *) NULL)
+    return(MagickFalse);
+  write_info=DestroyImageInfo(write_info);
+  if (WriteBlob(image,length,group4) != (ssize_t) length)
+    status=MagickFalse;
+  group4=(unsigned char *) RelinquishMagickMemory(group4);
+  return(status);
+}
+
 static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
 {
 #define CFormat  "/Filter [ /%s ]\n"
@@ -934,7 +1003,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     length;
 
   struct tm
-    *time_meridian;
+    local_time;
 
   time_t
     seconds;
@@ -1028,11 +1097,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,"<<\n");
       (void) WriteBlobString(image,"/Subtype /XML\n");
       *modify_date='\0';
-      value=GetImageProperty(image,"modify-date");
+      value=GetImageProperty(image,"date:modify");
       if (value != (const char *) NULL)
         (void) CopyMagickString(modify_date,value,MaxTextExtent);
       *create_date='\0';
-      value=GetImageProperty(image,"create-date");
+      value=GetImageProperty(image,"date:create");
       if (value != (const char *) NULL)
         (void) CopyMagickString(create_date,value,MaxTextExtent);
       (void) FormatMagickTime(time((time_t *) NULL),MaxTextExtent,timestamp);
@@ -1088,7 +1157,9 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   scene=0;
   do
   {
-    compression=image_info->compression;
+    compression=image->compression;
+    if (image_info->compression != UndefinedCompression)
+      compression=image_info->compression;
     switch (compression)
     {
       case FaxCompression:
@@ -1147,22 +1218,23 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     if (compression == JPEG2000Compression)
       {
         if (image->colorspace != RGBColorspace)
-          (void) SetImageColorspace(image,RGBColorspace);
+          (void) TransformImageColorspace(image,RGBColorspace);
       }
     /*
       Scale relative to dots-per-inch.
     */
     delta.x=DefaultResolution;
     delta.y=DefaultResolution;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    resolution.x=geometry_info.rho;
-    resolution.y=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      resolution.y=resolution.x;
-    if (image->x_resolution != 0.0)
-      resolution.x=image->x_resolution;
-    if (image->y_resolution != 0.0)
-      resolution.y=image->y_resolution;
+    resolution.x=image->x_resolution;
+    resolution.y=image->y_resolution;
+    if ((resolution.x == 0.0) || (resolution.y == 0.0))
+      {
+        flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          resolution.y=resolution.x;
+      }
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
@@ -1193,11 +1265,12 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     scale.x=(double) (geometry.width*delta.x)/resolution.x;
-    geometry.width=(unsigned long) (scale.x+0.5);
+    geometry.width=(unsigned long) floor(scale.x+0.5);
     scale.y=(double) (geometry.height*delta.y)/resolution.y;
-    geometry.height=(unsigned long) (scale.y+0.5);
+    geometry.height=(unsigned long) floor(scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info,
+      &image->exception);
     if (image->gravity != UndefinedGravity)
       {
         geometry.x=(-page_info.x);
@@ -1238,12 +1311,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,"/ProcSet %lu 0 R >>\n",
       object+3);
     (void) WriteBlobString(image,buffer);
-    (void) FormatMagickString(buffer,MaxTextExtent,"/MediaBox [0 0 %lu %lu]\n",
-      media_info.width,media_info.height);
+    (void) FormatMagickString(buffer,MaxTextExtent,
+      "/MediaBox [0 0 %g %g]\n",72.0*media_info.width/resolution.x,
+      72.0*media_info.height/resolution.y);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,
-      "/CropBox [%ld %ld %lu %lu]\n",geometry.x,geometry.y,geometry.x+
-      geometry.width,geometry.y+geometry.height);
+      "/CropBox [0 0 %g %g]\n",72.0*media_info.width/resolution.x,
+      72.0*media_info.height/resolution.y);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/Contents %lu 0 R\n",
       object+1);
@@ -1282,8 +1356,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,"ET\n");
         labels[i]=DestroyString(labels[i]);
       }
-    (void) FormatMagickString(buffer,MaxTextExtent,"%g 0 0 %g %ld %ld cm\n",
-      scale.x,scale.y,geometry.x,geometry.y);
+    (void) FormatMagickString(buffer,MaxTextExtent,
+      "%g 0 0 %g %ld %ld cm\n",scale.x,scale.y,geometry.x,geometry.y);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/Im%lu Do\n",image->scene);
     (void) WriteBlobString(image,buffer);
@@ -1378,13 +1452,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       }
       case LZWCompression:
       {
-         (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"LZWDecode");
-         break;
+        (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"LZWDecode");
+        break;
       }
       case ZipCompression:
       {
-         (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"FlateDecode");
-         break;
+        (void) FormatMagickString(buffer,MaxTextExtent,CFormat,"FlateDecode");
+        break;
       }
       case FaxCompression:
       case Group4Compression:
@@ -1392,9 +1466,9 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         (void) CopyMagickString(buffer,"/Filter [ /CCITTFaxDecode ]\n",
           MaxTextExtent);
         (void) WriteBlobString(image,buffer);
-        (void) FormatMagickString(buffer,MaxTextExtent,
-          "/DecodeParms [ << /K %s /Columns %ld /Rows %ld >> ]\n",
-          CCITTParam,image->columns,image->rows);
+        (void) FormatMagickString(buffer,MaxTextExtent,"/DecodeParms [ << "
+          "/K %s /BlackIs1 false /Columns %ld /Rows %ld >> ]\n",CCITTParam,
+          image->columns,image->rows);
         break;
       }
       default:
@@ -1443,22 +1517,24 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
           {
             if (LocaleCompare(CCITTParam,"0") == 0)
               {
-                (void) HuffmanEncodeImage(image_info,image);
+                (void) HuffmanEncodeImage(image_info,image,image);
                 break;
               }
-            (void) Huffman2DEncodeImage(image_info,image);
+            (void) Huffman2DEncodeImage(image_info,image,image);
             break;
           }
           case JPEGCompression:
           {
-            status=InjectImageBlob(image_info,image,"jpeg");
+            status=InjectImageBlob(image_info,image,image,"jpeg",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,image->exception.reason);
             break;
           }
           case JPEG2000Compression:
           {
-            status=InjectImageBlob(image_info,image,"jp2");
+            status=InjectImageBlob(image_info,image,image,"jp2",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,image->exception.reason);
             break;
@@ -1551,14 +1627,16 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         {
           case JPEGCompression:
           {
-            status=InjectImageBlob(image_info,image,"jpeg");
+            status=InjectImageBlob(image_info,image,image,"jpeg",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,image->exception.reason);
             break;
           }
           case JPEG2000Compression:
           {
-            status=InjectImageBlob(image_info,image,"jp2");
+            status=InjectImageBlob(image_info,image,image,"jp2",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,image->exception.reason);
             break;
@@ -1588,9 +1666,9 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               indexes=GetVirtualIndexQueue(image);
               for (x=0; x < (long) image->columns; x++)
               {
-                *q++=ScaleQuantumToChar(p->red);
-                *q++=ScaleQuantumToChar(p->green);
-                *q++=ScaleQuantumToChar(p->blue);
+                *q++=ScaleQuantumToChar(GetRedPixelComponent(p));
+                *q++=ScaleQuantumToChar(GetGreenPixelComponent(p));
+                *q++=ScaleQuantumToChar(GetBluePixelComponent(p));
                 if (image->colorspace == CMYKColorspace)
                   *q++=ScaleQuantumToChar(indexes[x]);
                 p++;
@@ -1634,9 +1712,9 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               indexes=GetVirtualIndexQueue(image);
               for (x=0; x < (long) image->columns; x++)
               {
-                Ascii85Encode(image,ScaleQuantumToChar(p->red));
-                Ascii85Encode(image,ScaleQuantumToChar(p->green));
-                Ascii85Encode(image,ScaleQuantumToChar(p->blue));
+                Ascii85Encode(image,ScaleQuantumToChar(GetRedPixelComponent(p)));
+                Ascii85Encode(image,ScaleQuantumToChar(GetGreenPixelComponent(p)));
+                Ascii85Encode(image,ScaleQuantumToChar(GetBluePixelComponent(p)));
                 if (image->colorspace == CMYKColorspace)
                   Ascii85Encode(image,ScaleQuantumToChar(indexes[x]));
                 p++;
@@ -1830,9 +1908,9 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         (void) CopyMagickString(buffer,"/Filter [ /CCITTFaxDecode ]\n",
           MaxTextExtent);
         (void) WriteBlobString(image,buffer);
-        (void) FormatMagickString(buffer,MaxTextExtent,
-          "/DecodeParms [ << /K %s /Columns %lu /Rows %lu >> ]\n",
-          CCITTParam,tile_image->columns,tile_image->rows);
+        (void) FormatMagickString(buffer,MaxTextExtent,"/DecodeParms [ << "
+          "/K %s /BlackIs1 false /Columns %lu /Rows %lu >> ]\n",CCITTParam,
+          tile_image->columns,tile_image->rows);
         break;
       }
       default:
@@ -1875,22 +1953,24 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
           {
             if (LocaleCompare(CCITTParam,"0") == 0)
               {
-                (void) HuffmanEncodeImage(image_info,tile_image);
+                (void) HuffmanEncodeImage(image_info,image,tile_image);
                 break;
               }
-            (void) Huffman2DEncodeImage(image_info,tile_image);
+            (void) Huffman2DEncodeImage(image_info,image,tile_image);
             break;
           }
           case JPEGCompression:
           {
-            status=InjectImageBlob(image_info,tile_image,"jpeg");
+            status=InjectImageBlob(image_info,image,tile_image,"jpeg",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,tile_image->exception.reason);
             break;
           }
           case JPEG2000Compression:
           {
-            status=InjectImageBlob(image_info,tile_image,"jp2");
+            status=InjectImageBlob(image_info,image,tile_image,"jp2",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,tile_image->exception.reason);
             break;
@@ -1975,14 +2055,16 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         {
           case JPEGCompression:
           {
-            status=InjectImageBlob(image_info,tile_image,"jpeg");
+            status=InjectImageBlob(image_info,image,tile_image,"jpeg",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,tile_image->exception.reason);
             break;
           }
           case JPEG2000Compression:
           {
-            status=InjectImageBlob(image_info,tile_image,"jp2");
+            status=InjectImageBlob(image_info,image,tile_image,"jp2",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,tile_image->exception.reason);
             break;
@@ -2016,9 +2098,9 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               indexes=GetVirtualIndexQueue(tile_image);
               for (x=0; x < (long) tile_image->columns; x++)
               {
-                *q++=ScaleQuantumToChar(p->red);
-                *q++=ScaleQuantumToChar(p->green);
-                *q++=ScaleQuantumToChar(p->blue);
+                *q++=ScaleQuantumToChar(GetRedPixelComponent(p));
+                *q++=ScaleQuantumToChar(GetGreenPixelComponent(p));
+                *q++=ScaleQuantumToChar(GetBluePixelComponent(p));
                 if (image->colorspace == CMYKColorspace)
                   *q++=ScaleQuantumToChar(indexes[x]);
                 p++;
@@ -2056,9 +2138,9 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               indexes=GetVirtualIndexQueue(tile_image);
               for (x=0; x < (long) tile_image->columns; x++)
               {
-                Ascii85Encode(image,ScaleQuantumToChar(p->red));
-                Ascii85Encode(image,ScaleQuantumToChar(p->green));
-                Ascii85Encode(image,ScaleQuantumToChar(p->blue));
+                Ascii85Encode(image,ScaleQuantumToChar(GetRedPixelComponent(p)));
+                Ascii85Encode(image,ScaleQuantumToChar(GetGreenPixelComponent(p)));
+                Ascii85Encode(image,ScaleQuantumToChar(GetBluePixelComponent(p)));
                 if (image->colorspace == CMYKColorspace)
                   Ascii85Encode(image,ScaleQuantumToChar(indexes[x]));
                 p++;
@@ -2298,7 +2380,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                *q++=ScaleQuantumToChar((Quantum) (QuantumRange-p->opacity));
+                *q++=ScaleQuantumToChar((Quantum) (GetAlphaPixelComponent(p)));
                 p++;
               }
             }
@@ -2334,7 +2416,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               for (x=0; x < (long) image->columns; x++)
               {
                 Ascii85Encode(image,ScaleQuantumToChar((Quantum) (QuantumRange-
-                  p->opacity)));
+                  GetOpacityPixelComponent(p))));
                 p++;
               }
             }
@@ -2376,10 +2458,14 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     EscapeParenthesis(image->filename));
   (void) WriteBlobString(image,buffer);
   seconds=time((time_t *) NULL);
-  time_meridian=localtime(&seconds);
+#if defined(MAGICKCORE_HAVE_LOCALTIME_R)
+  (void) localtime_r(&seconds,&local_time);
+#else
+  (void) memcpy(&local_time,localtime(&seconds),sizeof(local_time));
+#endif
   (void) FormatMagickString(date,MaxTextExtent,"D:%04d%02d%02d%02d%02d%02d",
-    time_meridian->tm_year+1900,time_meridian->tm_mon+1,time_meridian->tm_mday,
-    time_meridian->tm_hour,time_meridian->tm_min,time_meridian->tm_sec);
+    local_time.tm_year+1900,local_time.tm_mon+1,local_time.tm_mday,
+    local_time.tm_hour,local_time.tm_min,local_time.tm_sec);
   (void) FormatMagickString(buffer,MaxTextExtent,"/CreationDate (%s)\n",date);
   (void) WriteBlobString(image,buffer);
   (void) FormatMagickString(buffer,MaxTextExtent,"/ModDate (%s)\n",date);
