@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -90,13 +90,14 @@ static MagickBooleanType
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  InvokePostscriptDelegate() executes the postscript interpreter with the
+%  InvokePostscriptDelegate() executes the Postscript interpreter with the
 %  specified command.
 %
 %  The format of the InvokePostscriptDelegate method is:
 %
 %      MagickBooleanType InvokePostscriptDelegate(
-%        const MagickBooleanType verbose,const char *command)
+%        const MagickBooleanType verbose,const char *command,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -106,76 +107,96 @@ static MagickBooleanType
 %    o command: the address of a character string containing the command to
 %      execute.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 static MagickBooleanType InvokePostscriptDelegate(
-  const MagickBooleanType verbose,const char *command)
+  const MagickBooleanType verbose,const char *command,ExceptionInfo *exception)
 {
+  int
+    status;
+
 #if defined(MAGICKCORE_GS_DELEGATE) || defined(__WINDOWS__)
   char
     **argv;
 
-  const GhostscriptVectors
-    *gs_func;
+  const GhostInfo
+    *ghost_info;
 
   gs_main_instance
     *interpreter;
 
   int
     argc,
-    code,
-    status;
+    code;
 
   register long
     i;
 
 #if defined(__WINDOWS__)
-  gs_func=NTGhostscriptDLLVectors();
+  ghost_info=NTGhostscriptDLLVectors();
 #else
-  GhostscriptVectors
-    gs_func_struct;
+  GhostInfo
+    ghost_info_struct;
 
-  gs_func=(&gs_func_struct);
-  (void) ResetMagickMemory(&gs_func,0,sizeof(gs_func));
-  gs_func_struct.new_instance=(int (*)(gs_main_instance **,void *))
+  ghost_info=(&ghost_info_struct);
+  (void) ResetMagickMemory(&ghost_info,0,sizeof(ghost_info));
+  ghost_info_struct.new_instance=(int (*)(gs_main_instance **,void *))
     gsapi_new_instance;
-  gs_func_struct.init_with_args=(int (*)(gs_main_instance *,int,char **))
+  ghost_info_struct.init_with_args=(int (*)(gs_main_instance *,int,char **))
     gsapi_init_with_args;
-  gs_func_struct.run_string=(int (*)(gs_main_instance *,const char *,int,int *))
-    gsapi_run_string;
-  gs_func_struct.delete_instance=(void (*)(gs_main_instance *))
+  ghost_info_struct.run_string=(int (*)(gs_main_instance *,const char *,int,
+    int *)) gsapi_run_string;
+  ghost_info_struct.delete_instance=(void (*)(gs_main_instance *))
     gsapi_delete_instance;
-  gs_func_struct.exit=(int (*)(gs_main_instance *)) gsapi_exit;
+  ghost_info_struct.exit=(int (*)(gs_main_instance *)) gsapi_exit;
 #endif
-  if (gs_func == (GhostscriptVectors *) NULL)
-    return(SystemCommand(verbose,command) == 0 ? MagickFalse : MagickTrue);
+  if (ghost_info == (GhostInfo *) NULL)
+    {
+      status=SystemCommand(MagickFalse,verbose,command,exception);
+      return(status == 0 ? MagickTrue : MagickFalse);
+    }
   if (verbose != MagickFalse)
     {
       (void) fputs("[ghostscript library]",stdout);
       (void) fputs(strchr(command,' '),stdout);
     }
-  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
+  status=(ghost_info->new_instance)(&interpreter,(void *) NULL);
   if (status < 0)
-    return(SystemCommand(verbose,command) == 0 ? MagickFalse : MagickTrue);
+    {
+      status=SystemCommand(MagickFalse,verbose,command,exception);
+      return(status == 0 ? MagickTrue : MagickFalse);
+    }
   argv=StringToArgv(command,&argc);
-  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
+  status=(ghost_info->init_with_args)(interpreter,argc-1,argv+1);
   if (status == 0)
-    status=(gs_func->run_string)(interpreter,"systemdict /start get exec\n",0,
-      &code);
-  (gs_func->exit)(interpreter);
-  (gs_func->delete_instance)(interpreter);
+    status=(ghost_info->run_string)(interpreter,"systemdict /start get exec\n",
+      0,&code);
+  (ghost_info->exit)(interpreter);
+  (ghost_info->delete_instance)(interpreter);
 #if defined(__WINDOWS__)
   NTGhostscriptUnLoadDLL();
 #endif
   for (i=0; i < (long) argc; i++)
     argv[i]=DestroyString(argv[i]);
   argv=(char **) RelinquishMagickMemory(argv);
-  if ((status == 0) || (status == -101))
-    return(MagickFalse);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-    "Ghostscript returns status %d, exit code %d",status,code);
+  if ((status != 0) && (status != -101))
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      (void) ThrowMagickException(exception,GetMagickModule(),DelegateError,
+        "`%s': %s",command,message);
+      message=DestroyString(message);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Ghostscript returns status %d, exit code %d",status,code);
+      return(MagickFalse);
+    }
   return(MagickTrue);
 #else
-  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
+  status=SystemCommand(MagickFalse,verbose,command,exception);
+  return(status == 0 ? MagickTrue : MagickFalse);
 #endif
 }
 
@@ -199,8 +220,7 @@ static MagickBooleanType InvokePostscriptDelegate(
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -294,14 +314,16 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define BoundingBox  "BoundingBox:"
 #define BeginDocument  "BeginDocument:"
-#define BeginXMLCode  "begin_xml_code"
-#define EndXMLCode  "end_xml_code"
+#define BeginXMPPacket  "<?xpacket begin="
+#define EndXMPPacket  "<?xpacket end="
 #define ICCProfile "BeginICCProfile:"
 #define CMYKCustomColor  "CMYKCustomColor:"
+#define CMYKProcessColor  "CMYKProcessColor:"
 #define DocumentMedia  "DocumentMedia:"
 #define DocumentCustomColors  "DocumentCustomColors:"
 #define DocumentProcessColors  "DocumentProcessColors:"
 #define EndDocument  "EndDocument:"
+#define ICCBased  "ICCBased"
 #define HiResBoundingBox  "HiResBoundingBox:"
 #define ImageData  "ImageData:"
 #define PageBoundingBox  "PageBoundingBox:"
@@ -334,7 +356,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   Image
     *image,
-    *next_image,
+    *next,
     *postscript_image;
 
   ImageInfo
@@ -542,17 +564,19 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         profile=DestroyStringInfo(profile);
         continue;
       }
-    if (LocaleNCompare(BeginXMLCode,command,strlen(BeginXMLCode)) == 0)
+    if (LocaleNCompare(BeginXMPPacket,command,strlen(BeginXMPPacket)) == 0)
       {
+        register size_t
+          i;
+
         /*
-          Read XML code.
+          Read XMP profile.
         */
-        c=ReadBlobByte(image);
         p=command;
-        profile=AcquireStringInfo(0);
-        for (i=0; c != EOF; i++)
+        profile=StringToStringInfo(command);
+        for (i=GetStringInfoLength(profile)-1; c != EOF; i++)
         {
-          SetStringInfoLength(profile,(size_t) i+1);
+          SetStringInfoLength(profile,i+1);
           c=ReadBlobByte(image);
           GetStringInfoDatum(profile)[i]=(unsigned char) c;
           *p++=(char) c;
@@ -561,10 +585,11 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
             continue;
           *p='\0';
           p=command;
-          if (LocaleNCompare(EndXMLCode,command,strlen(EndXMLCode)) == 0)
+          if (LocaleNCompare(EndXMPPacket,command,strlen(EndXMPPacket)) == 0)
             break;
         }
-        (void) SetImageProfile(image,"xml-code",profile);
+        SetStringInfoLength(profile,i);
+        (void) SetImageProfile(image,"xmp",profile);
         profile=DestroyStringInfo(profile);
         continue;
       }
@@ -573,9 +598,18 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     length=strlen(DocumentProcessColors);
     if (LocaleNCompare(DocumentProcessColors,command,length) == 0)
-      cmyk=MagickTrue;
+      {
+        if ((GlobExpression(command,"*Cyan*",MagickTrue) != MagickFalse) ||
+            (GlobExpression(command,"*Magenta*",MagickTrue) != MagickFalse) ||
+            (GlobExpression(command,"*Yellow*",MagickTrue) != MagickFalse))
+          cmyk=MagickTrue;
+      }
     if (LocaleNCompare(CMYKCustomColor,command,strlen(CMYKCustomColor)) == 0)
       cmyk=MagickTrue;
+    if (LocaleNCompare(CMYKProcessColor,command,strlen(CMYKProcessColor)) == 0)
+      cmyk=MagickTrue;
+    if (LocaleNCompare(ICCBased,command,strlen(ICCBased)) == 0)
+      cmyk=MagickTrue;
     length=strlen(DocumentCustomColors);
     if ((LocaleNCompare(DocumentCustomColors,command,length) == 0) ||
         (LocaleNCompare(CMYKCustomColor,command,strlen(CMYKCustomColor)) == 0) ||
@@ -631,11 +665,12 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Set Postscript render geometry.
         */
-        (void) FormatMagickString(geometry,MaxTextExtent,"%gx%g%+g%+g",
-          bounds.x2-bounds.x1,bounds.y2-bounds.y1,bounds.x1,bounds.y1);
+        (void) FormatMagickString(geometry,MaxTextExtent,
+          "%gx%g%+.15g%+.15g",bounds.x2-bounds.x1,bounds.y2-bounds.y1,
+          bounds.x1,bounds.y1);
         (void) SetImageProperty(image,"ps:HiResBoundingBox",geometry);
-        page.width=(unsigned long) (bounds.x2-bounds.x1+0.5);
-        page.height=(unsigned long) (bounds.y2-bounds.y1+0.5);
+        page.width=(unsigned long) floor(bounds.x2-bounds.x1+0.5);
+        page.height=(unsigned long) floor(bounds.y2-bounds.y1+0.5);
         hires_bounds=bounds;
       }
   }
@@ -656,10 +691,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) CopyMagickString(command,"/setpagedevice {pop} bind 1 index where {"
     "dup wcheck {3 1 roll put} {pop def} ifelse} {def} ifelse\n"
     "<</UseCIEColor true>>setpagedevice\n",MaxTextExtent);
-  status=write(file,command,(unsigned int) strlen(command));
+  count=write(file,command,(unsigned int) strlen(command));
   (void) FormatMagickString(translate_geometry,MaxTextExtent,
     "%g %g translate\n",-bounds.x1,-bounds.y1);
-  status=write(file,translate_geometry,strlen(translate_geometry));
+  count=write(file,translate_geometry,strlen(translate_geometry));
   file=close(file)-1;
   /*
     Render Postscript with the Ghostscript delegate.
@@ -685,10 +720,11 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ParseAbsoluteGeometry(PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
-  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",image->x_resolution,
-    image->y_resolution);
-  page.width=(unsigned long) (page.width*image->x_resolution/delta.x+0.5);
-  page.height=(unsigned long) (page.height*image->y_resolution/delta.y+0.5);
+  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",
+    image->x_resolution,image->y_resolution);
+  page.width=(unsigned long) floor(page.width*image->x_resolution/delta.x+0.5);
+  page.height=(unsigned long) floor(page.height*image->y_resolution/delta.y+
+    0.5);
   (void) FormatMagickString(options,MaxTextExtent,"-g%lux%lu ",
     page.width,page.height);
   read_info=CloneImageInfo(image_info);
@@ -716,15 +752,15 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,density,options,
     read_info->filename,postscript_filename,input_filename);
-  status=InvokePostscriptDelegate(read_info->verbose,command);
-  if ((status != MagickFalse) ||
+  status=InvokePostscriptDelegate(read_info->verbose,command,exception);
+  if ((status == MagickFalse) ||
       (IsPostscriptRendered(read_info->filename) == MagickFalse))
     {
       (void) ConcatenateMagickString(command," -c showpage",MaxTextExtent);
-      status=InvokePostscriptDelegate(read_info->verbose,command);
+      status=InvokePostscriptDelegate(read_info->verbose,command,exception);
     }
   postscript_image=(Image *) NULL;
-  if (status == MagickFalse)
+  if (status != MagickFalse)
     postscript_image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(postscript_filename);
   (void) RelinquishUniqueFileResource(read_info->filename);
@@ -767,7 +803,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           PrependImageToList(&postscript_image,clone_image);
       }
     }
-  scene=0;
   do
   {
     (void) CopyMagickString(postscript_image->filename,filename,MaxTextExtent);
@@ -776,14 +811,19 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (rows != 0)
       postscript_image->magick_rows=rows;
     postscript_image->page=page;
-    postscript_image->scene=scene++;
     (void) CloneImageProfiles(postscript_image,image);
     (void) CloneImageProperties(postscript_image,image);
-    next_image=SyncNextImageInList(postscript_image);
-    if (next_image != (Image *) NULL)
-      postscript_image=next_image;
-  } while (next_image != (Image *) NULL);
+    next=SyncNextImageInList(postscript_image);
+    if (next != (Image *) NULL)
+      postscript_image=next;
+  } while (next != (Image *) NULL);
   image=DestroyImageList(image);
+  scene=0;
+  for (next=GetFirstImageInList(postscript_image); next != (Image *) NULL; )
+  {
+    next->scene=scene++;
+    next=GetNextImageInList(next);
+  }
   return(GetFirstImageInList(postscript_image));
 }
 
@@ -971,27 +1011,27 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
   static const char
     *hex_digits[] =
     {
-      "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "0A", "0B", 
-      "0C", "0D", "0E", "0F", "10", "11", "12", "13", "14", "15", "16", "17", 
-      "18", "19", "1A", "1B", "1C", "1D", "1E", "1F", "20", "21", "22", "23", 
-      "24", "25", "26", "27", "28", "29", "2A", "2B", "2C", "2D", "2E", "2F", 
-      "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "3A", "3B", 
-      "3C", "3D", "3E", "3F", "40", "41", "42", "43", "44", "45", "46", "47", 
-      "48", "49", "4A", "4B", "4C", "4D", "4E", "4F", "50", "51", "52", "53", 
-      "54", "55", "56", "57", "58", "59", "5A", "5B", "5C", "5D", "5E", "5F", 
-      "60", "61", "62", "63", "64", "65", "66", "67", "68", "69", "6A", "6B", 
-      "6C", "6D", "6E", "6F", "70", "71", "72", "73", "74", "75", "76", "77", 
-      "78", "79", "7A", "7B", "7C", "7D", "7E", "7F", "80", "81", "82", "83", 
-      "84", "85", "86", "87", "88", "89", "8A", "8B", "8C", "8D", "8E", "8F", 
-      "90", "91", "92", "93", "94", "95", "96", "97", "98", "99", "9A", "9B", 
-      "9C", "9D", "9E", "9F", "A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", 
-      "A8", "A9", "AA", "AB", "AC", "AD", "AE", "AF", "B0", "B1", "B2", "B3", 
-      "B4", "B5", "B6", "B7", "B8", "B9", "BA", "BB", "BC", "BD", "BE", "BF", 
-      "C0", "C1", "C2", "C3", "C4", "C5", "C6", "C7", "C8", "C9", "CA", "CB", 
-      "CC", "CD", "CE", "CF", "D0", "D1", "D2", "D3", "D4", "D5", "D6", "D7", 
-      "D8", "D9", "DA", "DB", "DC", "DD", "DE", "DF", "E0", "E1", "E2", "E3", 
-      "E4", "E5", "E6", "E7", "E8", "E9", "EA", "EB", "EC", "ED", "EE", "EF", 
-      "F0", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "FA", "FB", 
+      "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "0A", "0B",
+      "0C", "0D", "0E", "0F", "10", "11", "12", "13", "14", "15", "16", "17",
+      "18", "19", "1A", "1B", "1C", "1D", "1E", "1F", "20", "21", "22", "23",
+      "24", "25", "26", "27", "28", "29", "2A", "2B", "2C", "2D", "2E", "2F",
+      "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "3A", "3B",
+      "3C", "3D", "3E", "3F", "40", "41", "42", "43", "44", "45", "46", "47",
+      "48", "49", "4A", "4B", "4C", "4D", "4E", "4F", "50", "51", "52", "53",
+      "54", "55", "56", "57", "58", "59", "5A", "5B", "5C", "5D", "5E", "5F",
+      "60", "61", "62", "63", "64", "65", "66", "67", "68", "69", "6A", "6B",
+      "6C", "6D", "6E", "6F", "70", "71", "72", "73", "74", "75", "76", "77",
+      "78", "79", "7A", "7B", "7C", "7D", "7E", "7F", "80", "81", "82", "83",
+      "84", "85", "86", "87", "88", "89", "8A", "8B", "8C", "8D", "8E", "8F",
+      "90", "91", "92", "93", "94", "95", "96", "97", "98", "99", "9A", "9B",
+      "9C", "9D", "9E", "9F", "A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7",
+      "A8", "A9", "AA", "AB", "AC", "AD", "AE", "AF", "B0", "B1", "B2", "B3",
+      "B4", "B5", "B6", "B7", "B8", "B9", "BA", "BB", "BC", "BD", "BE", "BF",
+      "C0", "C1", "C2", "C3", "C4", "C5", "C6", "C7", "C8", "C9", "CA", "CB",
+      "CC", "CD", "CE", "CF", "D0", "D1", "D2", "D3", "D4", "D5", "D6", "D7",
+      "D8", "D9", "DA", "DB", "DC", "DD", "DE", "DF", "E0", "E1", "E2", "E3",
+      "E4", "E5", "E6", "E7", "E8", "E9", "EA", "EB", "EC", "ED", "EE", "EF",
+      "F0", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "FA", "FB",
       "FC", "FD", "FE", "FF",  (char *) NULL
     },
     *PostscriptProlog[]=
@@ -1247,7 +1287,6 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       "  currentfile buffer readline pop",
       "  token pop /compression exch def pop",
       "  class 0 gt { PseudoClassImage } { DirectClassImage } ifelse",
-      "  grestore",
       (char *) NULL
     };
 
@@ -1352,18 +1391,19 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     */
     if ((image->colorspace != RGBColorspace) &&
         (image->colorspace != CMYKColorspace))
-      (void) SetImageColorspace(image,RGBColorspace);
+      (void) TransformImageColorspace(image,RGBColorspace);
     delta.x=DefaultResolution;
     delta.y=DefaultResolution;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    resolution.x=geometry_info.rho;
-    resolution.y=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      resolution.y=resolution.x;
-    if (image->x_resolution != 0.0)
-      resolution.x=image->x_resolution;
-    if (image->y_resolution != 0.0)
-      resolution.y=image->y_resolution;
+    resolution.x=image->x_resolution;
+    resolution.y=image->y_resolution;
+    if ((resolution.x == 0.0) || (resolution.y == 0.0))
+      {
+        flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          resolution.y=resolution.x;
+      }
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
@@ -1394,9 +1434,9 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     scale.x=(double) (geometry.width*delta.x)/resolution.x;
-    geometry.width=(unsigned long) (scale.x+0.5);
+    geometry.width=(unsigned long) floor(scale.x+0.5);
     scale.y=(double) (geometry.height*delta.y)/resolution.y;
-    geometry.height=(unsigned long) (scale.y+0.5);
+    geometry.height=(unsigned long) floor(scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info,
       &image->exception);
@@ -1428,9 +1468,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
           image->filename);
         (void) WriteBlobString(image,buffer);
         timer=time((time_t *) NULL);
-        (void) localtime(&timer);
-        (void) CopyMagickString(date,ctime(&timer),MaxTextExtent);
-        date[strlen(date)-1]='\0';
+        (void) FormatMagickTime(timer,MaxTextExtent,date);
         (void) FormatMagickString(buffer,MaxTextExtent,
           "%%%%CreationDate: (%s)\n",date);
         (void) WriteBlobString(image,buffer);
@@ -1445,13 +1483,13 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
         else
           {
             (void) FormatMagickString(buffer,MaxTextExtent,
-              "%%%%BoundingBox: %ld %ld %ld %ld\n",(long) (bounds.x1+0.5),
-              (long) (bounds.y1+0.5),(long) (bounds.x2+0.5),
-              (long) (bounds.y2+0.5));
+              "%%%%BoundingBox: %ld %ld %ld %ld\n",(long) ceil(bounds.x1-0.5),
+              (long) ceil(bounds.y1-0.5),(long) floor(bounds.x2+0.5),
+              (long) floor(bounds.y2+0.5));
             (void) WriteBlobString(image,buffer);
             (void) FormatMagickString(buffer,MaxTextExtent,
-              "%%%%HiResBoundingBox: %g %g %g %g\n",bounds.x1,bounds.y1,
-              bounds.x2,bounds.y2);
+              "%%%%HiResBoundingBox: %g %g %g %g\n",bounds.x1,
+              bounds.y1,bounds.x2,bounds.y2);
           }
         (void) WriteBlobString(image,buffer);
         profile=GetImageProfile(image,"8bim");
@@ -1474,16 +1512,20 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
             }
             (void) WriteBlobString(image,"\n%EndPhotoshop\n");
           }
-        profile=GetImageProfile(image,"xml-code");
-        if (profile != (StringInfo *) NULL)
+        profile=GetImageProfile(image,"xmp");
+        if (0 && (profile != (StringInfo *) NULL))
           {
             /*
               Embed XML profile.
             */
             (void) WriteBlobString(image,"\n%begin_xml_code\n");
+            (void) FormatMagickString(buffer,MaxTextExtent,
+               "\n%%begin_xml_packet: %lu\n",(unsigned long)
+               GetStringInfoLength(profile));
+            (void) WriteBlobString(image,buffer);
             for (i=0; i < (long) GetStringInfoLength(profile); i++)
               (void) WriteBlobByte(image,GetStringInfoDatum(profile)[i]);
-            (void) WriteBlobString(image,"\n%end_xml_code\n");
+            (void) WriteBlobString(image,"\n%end_xml_packet\n%end_xml_code\n");
           }
         value=GetImageProperty(image,"label");
         if (value != (const char *) NULL)
@@ -1509,7 +1551,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,"\n%%BeginDefaults\n");
         (void) WriteBlobString(image,"%%EndDefaults\n\n");
         if ((LocaleCompare(image_info->magick,"EPI") == 0) ||
-            (LocaleCompare(image_info->magick,"EPSI") == 0))
+            (LocaleCompare(image_info->magick,"EPSI") == 0) ||
+            (LocaleCompare(image_info->magick,"EPT") == 0))
           {
             Image
               *preview_image;
@@ -1562,6 +1605,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                         *q++='\n';
                         (void) WriteBlob(image,q-pixels,pixels);
                         q=pixels;
+                        (void) WriteBlobString(image,"%  ");
                       };
                     bit=0;
                     byte=0;
@@ -1640,8 +1684,9 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     /*
       Output image data.
     */
-    (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld\n%g %g\n%f\n",
-      geometry.x,geometry.y,scale.x,scale.y,pointsize);
+    (void) FormatMagickString(buffer,MaxTextExtent,
+      "%ld %ld\n%g %g\n%g\n",geometry.x,geometry.y,scale.x,scale.y,
+      pointsize);
     (void) WriteBlobString(image,buffer);
     labels=(char **) NULL;
     value=GetImageProperty(image,"label");
@@ -1875,9 +1920,9 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                     }
                   else
                     {
-                      q=PopHexPixel(hex_digits,ScaleQuantumToChar(p->red),q);
-                      q=PopHexPixel(hex_digits,ScaleQuantumToChar(p->green),q);
-                      q=PopHexPixel(hex_digits,ScaleQuantumToChar(p->blue),q);
+                      q=PopHexPixel(hex_digits,ScaleQuantumToChar(GetRedPixelComponent(p)),q);
+                      q=PopHexPixel(hex_digits,ScaleQuantumToChar(GetGreenPixelComponent(p)),q);
+                      q=PopHexPixel(hex_digits,ScaleQuantumToChar(GetBluePixelComponent(p)),q);
                     }
                   if ((q-pixels+6) >= 80)
                     {
@@ -2044,12 +2089,13 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
   if (page > 2)
     {
       (void) FormatMagickString(buffer,MaxTextExtent,
-        "%%%%BoundingBox: %ld %ld %ld %ld\n",(long) (bounds.x1+0.5),
-        (long) (bounds.y1+0.5),(long) (bounds.x2+0.5),(long) (bounds.y2+0.5));
+        "%%%%BoundingBox: %ld %ld %ld %ld\n",(long) ceil(bounds.x1-0.5),
+        (long) ceil(bounds.y1-0.5),(long) floor(bounds.x2+0.5),(long)
+        floor(bounds.y2+0.5));
       (void) WriteBlobString(image,buffer);
       (void) FormatMagickString(buffer,MaxTextExtent,
-        "%%%%HiResBoundingBox: %g %g %g %g\n",bounds.x1,bounds.y1,bounds.x2,
-        bounds.y2);
+        "%%%%HiResBoundingBox: %g %g %g %g\n",bounds.x1,bounds.y1,
+        bounds.x2,bounds.y2);
       (void) WriteBlobString(image,buffer);
     }
   (void) WriteBlobString(image,"%%EOF\n");
