@@ -17,7 +17,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -55,8 +55,10 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/pixel.h"
+#include "magick/policy.h"
 #include "magick/profile.h"
 #include "magick/property.h"
 #include "magick/quantum.h"
@@ -66,6 +68,7 @@
 #include "magick/statistic.h"
 #include "magick/stream.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/timer.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
@@ -78,6 +81,55 @@ static SemaphoreInfo
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   C o n s t i t u t e C o m p o n e n t G e n e s i s                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ConstituteComponentGenesis() instantiates the constitute component.
+%
+%  The format of the ConstituteComponentGenesis method is:
+%
+%      MagickBooleanType ConstituteComponentGenesis(void)
+%
+*/
+MagickExport MagickBooleanType ConstituteComponentGenesis(void)
+{
+  AcquireSemaphoreInfo(&constitute_semaphore);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   C o n s t i t u t e C o m p o n e n t T e r m i n u s                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ConstituteComponentTerminus() destroys the constitute component.
+%
+%  The format of the ConstituteComponentTerminus method is:
+%
+%      ConstituteComponentTerminus(void)
+%
+*/
+MagickExport void ConstituteComponentTerminus(void)
+{
+  if (constitute_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&constitute_semaphore);
+  DestroySemaphoreInfo(&constitute_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   C o n s t i t u t e I m a g e                                             %
 %                                                                             %
 %                                                                             %
@@ -162,30 +214,6 @@ MagickExport Image *ConstituteImage(const unsigned long columns,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   D e s t r o y C o n s t i t u t e                                         %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  DestroyConstitute() destroys the constitute environment.
-%
-%  The format of the DestroyConstitute method is:
-%
-%      DestroyConstitute(void)
-%
-*/
-MagickExport void DestroyConstitute(void)
-{
-  if (constitute_semaphore != (SemaphoreInfo *) NULL)
-    DestroySemaphoreInfo(&constitute_semaphore);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   P i n g I m a g e                                                         %
 %                                                                             %
 %                                                                             %
@@ -298,46 +326,8 @@ MagickExport Image *PingImages(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  if (*image_info->filename == '@')
-    {
-      char
-        *command,
-        **image_list;
-
-      int
-        number_images;
-
-      register long
-        i;
-
-      /*
-        A file with a list of image filenames to read.
-      */
-      command=FileToString(image_info->filename+1,~0,exception);
-      if (command == (char *) NULL)
-        return((Image *) NULL);
-      StripString(command);
-      image_list=StringToArgv(command,&number_images);
-      command=DestroyString(command);
-      read_info=CloneImageInfo(image_info);
-      images=NewImageList();
-      for (i=1; i < number_images; i++)
-      {
-        (void) CopyMagickString(read_info->filename,image_list[i],
-          MaxTextExtent);
-        image=PingImage(read_info,exception);
-        if (image == (Image *) NULL)
-          continue;
-        AppendImageToList(&images,image);
-      }
-      read_info=DestroyImageInfo(read_info);
-      for (i=0; i < number_images; i++)
-        image_list[i]=DestroyString(image_list[i]);
-      image_list=(char **) RelinquishMagickMemory(image_list);
-      return(images);
-    }
-  (void) InterpretImageFilename(filename,MaxTextExtent,image_info->filename,
-    (int) image_info->scene);
+  (void) InterpretImageFilename(image_info,(Image *) NULL,image_info->filename,
+    (int) image_info->scene,filename);
   if (LocaleCompare(filename,image_info->filename) != 0)
     {
       ExceptionInfo
@@ -352,15 +342,15 @@ MagickExport Image *PingImages(const ImageInfo *image_info,
       */
       read_info=CloneImageInfo(image_info);
       sans=AcquireExceptionInfo();
-      (void) SetImageInfo(read_info,MagickFalse,sans);
+      (void) SetImageInfo(read_info,0,sans);
       sans=DestroyExceptionInfo(sans);
       (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
       images=NewImageList();
       extent=(long) (read_info->scene+read_info->number_scenes);
       for (scene=(long) read_info->scene; scene < (long) extent; scene++)
       {
-        (void) InterpretImageFilename(read_info->filename,MaxTextExtent,
-          filename,(int) scene);
+        (void) InterpretImageFilename(image_info,(Image *) NULL,filename,(int)
+          scene,read_info->filename);
         image=PingImage(read_info,exception);
         if (image == (Image *) NULL)
           continue;
@@ -418,7 +408,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     *magick_info;
 
   ExceptionInfo
-    sans_exception;
+    *sans_exception;
 
   GeometryInfo
     geometry_info;
@@ -434,6 +424,12 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     flags,
     thread_support;
 
+  PolicyDomain
+    domain;
+
+  PolicyRights
+    rights;
+
   /*
     Determine image type from filename prefix or suffix (e.g. image.jpg).
   */
@@ -446,14 +442,24 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   read_info=CloneImageInfo(image_info);
   (void) CopyMagickString(magick_filename,read_info->filename,MaxTextExtent);
-  (void) SetImageInfo(read_info,MagickFalse,exception);
+  (void) SetImageInfo(read_info,0,exception);
   (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
   (void) CopyMagickString(magick,read_info->magick,MaxTextExtent);
+  domain=CoderPolicyDomain;
+  rights=ReadPolicyRights;
+  if (IsRightsAuthorized(domain,rights,read_info->magick) == MagickFalse)
+    {
+      errno=EPERM;
+      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+        "NotAuthorized","`%s'",read_info->filename);
+      return((Image *) NULL);
+    }
   /*
     Call appropriate image reader based on image type.
   */
-  GetExceptionInfo(&sans_exception);
-  magick_info=GetMagickInfo(read_info->magick,&sans_exception);
+  sans_exception=AcquireExceptionInfo();
+  magick_info=GetMagickInfo(read_info->magick,sans_exception);
+  sans_exception=DestroyExceptionInfo(sans_exception);
   if (magick_info != (const MagickInfo *) NULL)
     {
       if (GetMagickEndianSupport(magick_info) == MagickFalse)
@@ -464,13 +470,12 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
           {
             unsigned long
               lsb_first;
-  
+
             lsb_first=1;
             read_info->endian=(*(char *) &lsb_first) == 1 ? LSBEndian :
               MSBEndian;
          }
     }
-  (void) DestroyExceptionInfo(&sans_exception);
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetMagickSeekableStream(magick_info) != MagickFalse))
     {
@@ -507,15 +512,17 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       image=DestroyImage(image);
     }
   image=NewImageList();
+  if (constitute_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&constitute_semaphore);
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetImageDecoder(magick_info) != (DecodeImageHandler *) NULL))
     {
-      thread_support=GetMagickThreadSupport(magick_info);      
+      thread_support=GetMagickThreadSupport(magick_info);
       if ((thread_support & DecoderThreadSupport) == 0)
-        AcquireSemaphoreInfo(&constitute_semaphore);
+        LockSemaphoreInfo(constitute_semaphore);
       image=GetImageDecoder(magick_info)(read_info,exception);
       if ((thread_support & DecoderThreadSupport) == 0)
-        RelinquishSemaphoreInfo(constitute_semaphore);
+        UnlockSemaphoreInfo(constitute_semaphore);
     }
   else
     {
@@ -544,14 +551,14 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         MaxTextExtent);
       *read_info->filename='\0';
       if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
-        AcquireSemaphoreInfo(&constitute_semaphore);
+        LockSemaphoreInfo(constitute_semaphore);
       (void) InvokeDelegate(read_info,image,read_info->magick,(char *) NULL,
         exception);
       if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
-        RelinquishSemaphoreInfo(constitute_semaphore);
+        UnlockSemaphoreInfo(constitute_semaphore);
       image=DestroyImageList(image);
       read_info->temporary=MagickTrue;
-      (void) SetImageInfo(read_info,MagickFalse,exception);
+      (void) SetImageInfo(read_info,0,exception);
       magick_info=GetMagickInfo(read_info->magick,exception);
       if ((magick_info == (const MagickInfo *) NULL) ||
           (GetImageDecoder(magick_info) == (DecodeImageHandler *) NULL))
@@ -566,12 +573,12 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
           read_info=DestroyImageInfo(read_info);
           return((Image *) NULL);
         }
-      thread_support=GetMagickThreadSupport(magick_info);      
+      thread_support=GetMagickThreadSupport(magick_info);
       if ((thread_support & DecoderThreadSupport) == 0)
-        AcquireSemaphoreInfo(&constitute_semaphore);
+        LockSemaphoreInfo(constitute_semaphore);
       image=(Image *) (GetImageDecoder(magick_info))(read_info,exception);
       if ((thread_support & DecoderThreadSupport) == 0)
-        RelinquishSemaphoreInfo(constitute_semaphore);
+        UnlockSemaphoreInfo(constitute_semaphore);
     }
   if (read_info->temporary != MagickFalse)
     {
@@ -635,10 +642,13 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       MaxTextExtent);
     if (IsBlobTemporary(image) != MagickFalse)
       (void) CopyMagickString(next->filename,filename,MaxTextExtent);
-    value=GetImageProperty(next,"exif:Orientation");
+    value=GetImageProperty(next,"tiff:Orientation");
+    if (value == (char *) NULL)
+      value=GetImageProperty(next,"exif:Orientation");
     if (value != (char *) NULL)
       {
-        next->orientation=(OrientationType) atol(value);
+        next->orientation=(OrientationType) StringToLong(value);
+        (void) DeleteImageProperty(next,"tiff:Orientation");
         (void) DeleteImageProperty(next,"exif:Orientation");
       }
     value=GetImageProperty(next,"tiff:XResolution");
@@ -672,7 +682,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       value=GetImageProperty(next,"exif:ResolutionUnit");
     if (value != (char *) NULL)
       {
-        next->units=(ResolutionType) (atoi(value)-1);
+        next->units=(ResolutionType) (StringToLong(value)-1);
         (void) DeleteImageProperty(next,"exif:ResolutionUnit");
         (void) DeleteImageProperty(next,"tiff:ResolutionUnit");
       }
@@ -682,21 +692,33 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       next->page.height=next->rows;
     if (*read_info->filename != '\0')
       {
+        char
+          *property;
+
         const char
           *option;
 
         option=GetImageOption(read_info,"caption");
         if (option != (const char *) NULL)
-          (void) SetImageProperty(next,"caption",InterpretImageProperties(
-            read_info,next,option));
+          {
+            property=InterpretImageProperties(read_info,next,option);
+            (void) SetImageProperty(next,"caption",property);
+            property=DestroyString(property);
+          }
         option=GetImageOption(read_info,"comment");
         if (option != (const char *) NULL)
-          (void) SetImageProperty(next,"comment",InterpretImageProperties(
-            read_info,next,option));
+          {
+            property=InterpretImageProperties(read_info,next,option);
+            (void) SetImageProperty(next,"comment",property);
+            property=DestroyString(property);
+          }
         option=GetImageOption(read_info,"label");
         if (option != (const char *) NULL)
-          (void) SetImageProperty(next,"label",InterpretImageProperties(
-            read_info,next,option));
+          {
+            property=InterpretImageProperties(read_info,next,option);
+            (void) SetImageProperty(next,"label",property);
+            property=DestroyString(property);
+          }
       }
     if (LocaleCompare(next->magick,"TEXT") == 0)
       (void) ParseAbsoluteGeometry("0x0+0+0",&next->page);
@@ -725,7 +747,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
                   Image
                     *size_image;
 
-                  flags=ParseSizeGeometry(next,read_info->extract,&geometry,
+                  flags=ParseRegionGeometry(next,read_info->extract,&geometry,
                     exception);
                   size_image=ResizeImage(next,geometry.width,geometry.height,
                     next->filter,next->blur,exception);
@@ -752,10 +774,10 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       }
     (void) FormatMagickTime(GetBlobProperties(next)->st_mtime,MaxTextExtent,
       timestamp);
-    (void) SetImageProperty(next,"modify-date",timestamp);
+    (void) SetImageProperty(next,"date:modify",timestamp);
     (void) FormatMagickTime(GetBlobProperties(next)->st_ctime,MaxTextExtent,
       timestamp);
-    (void) SetImageProperty(next,"create-date",timestamp);
+    (void) SetImageProperty(next,"date:create",timestamp);
     if (read_info->verbose != MagickFalse)
       (void) IdentifyImage(next,stdout,MagickFalse);
     image=next;
@@ -810,46 +832,8 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  if (*image_info->filename == '@')
-    {
-      char
-        *command,
-        **image_list;
-
-      int
-        number_images;
-
-      register long
-        i;
-
-      /*
-        A file with a list of image filenames to read.
-      */
-      command=FileToString(image_info->filename+1,~0,exception);
-      if (command == (char *) NULL)
-        return((Image *) NULL);
-      StripString(command);
-      image_list=StringToArgv(command,&number_images);
-      command=DestroyString(command);
-      read_info=CloneImageInfo(image_info);
-      images=NewImageList();
-      for (i=1; i < number_images; i++)
-      {
-        (void) CopyMagickString(read_info->filename,image_list[i],
-          MaxTextExtent);
-        image=ReadImage(read_info,exception);
-        if (image == (Image *) NULL)
-          continue;
-        AppendImageToList(&images,image);
-      }
-      read_info=DestroyImageInfo(read_info);
-      for (i=0; i < number_images; i++)
-        image_list[i]=DestroyString(image_list[i]);
-      image_list=(char **) RelinquishMagickMemory(image_list);
-      return(images);
-    }
-  (void) InterpretImageFilename(filename,MaxTextExtent,image_info->filename,
-    (int) image_info->scene);
+  (void) InterpretImageFilename(image_info,(Image *) NULL,image_info->filename,
+    (int) image_info->scene,filename);
   if (LocaleCompare(filename,image_info->filename) != 0)
     {
       ExceptionInfo
@@ -864,15 +848,15 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
       */
       read_info=CloneImageInfo(image_info);
       sans=AcquireExceptionInfo();
-      (void) SetImageInfo(read_info,MagickFalse,sans);
+      (void) SetImageInfo(read_info,0,sans);
       sans=DestroyExceptionInfo(sans);
       (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
       images=NewImageList();
       extent=(long) (read_info->scene+read_info->number_scenes);
-      for (scene=read_info->scene; scene < (long) extent; scene++)
+      for (scene=(long) read_info->scene; scene < (long) extent; scene++)
       {
-        (void) InterpretImageFilename(read_info->filename,MaxTextExtent,
-          filename,(int) scene);
+        (void) InterpretImageFilename(image_info,(Image *) NULL,filename,(int)
+          scene,read_info->filename);
         image=ReadImage(read_info,exception);
         if (image == (Image *) NULL)
           continue;
@@ -984,6 +968,9 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
   char
     filename[MaxTextExtent];
 
+  const char
+    *option;
+
   const DelegateInfo
     *delegate_info;
 
@@ -1003,6 +990,12 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
   MagickStatusType
     thread_support;
 
+  PolicyDomain
+    domain;
+
+  PolicyRights
+    rights;
+
   /*
     Determine image type from filename prefix or suffix (e.g. image.jpg).
   */
@@ -1018,7 +1011,7 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
   (void) CopyMagickString(write_info->filename,image->filename,MaxTextExtent);
   if (*write_info->magick == '\0')
     (void) CopyMagickString(write_info->magick,image->magick,MaxTextExtent);
-  (void) SetImageInfo(write_info,MagickTrue,sans_exception);
+  (void) SetImageInfo(write_info,1,sans_exception);
   if (LocaleCompare(write_info->magick,"clipmask") == 0)
     {
       if (image->clip_mask == (Image *) NULL)
@@ -1028,11 +1021,20 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
           return(MagickFalse);
         }
       image=image->clip_mask;
-      (void) SetImageInfo(write_info,MagickTrue,sans_exception);
+      (void) SetImageInfo(write_info,1,sans_exception);
     }
   (void) CopyMagickString(filename,image->filename,MaxTextExtent);
   (void) CopyMagickString(image->filename,write_info->filename,MaxTextExtent);
+  domain=CoderPolicyDomain;
+  rights=WritePolicyRights;
+  if (IsRightsAuthorized(domain,rights,write_info->magick) == MagickFalse)
+    {
+      sans_exception=DestroyExceptionInfo(sans_exception);
+      errno=EPERM;
+      ThrowBinaryException(PolicyError,"NotAuthorized",filename);
+    }
   magick_info=GetMagickInfo(write_info->magick,sans_exception);
+  sans_exception=DestroyExceptionInfo(sans_exception);
   if (magick_info != (const MagickInfo *) NULL)
     {
       if (GetMagickEndianSupport(magick_info) == MagickFalse)
@@ -1043,13 +1045,16 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
           {
             unsigned long
               lsb_first;
-  
+
             lsb_first=1;
             image->endian=(*(char *) &lsb_first) == 1 ? LSBEndian : MSBEndian;
          }
     }
   (void) SyncImageProfiles(image);
-  if ((write_info->page == (char *) NULL) &&
+  option=GetImageOption(image_info,"delegate:bimodal");
+  if ((option != (const char *) NULL) && 
+      (IsMagickTrue(option) != MagickFalse) &&
+      (write_info->page == (char *) NULL) &&
       (GetPreviousImageInList(image) == (Image *) NULL) &&
       (GetNextImageInList(image) == (Image *) NULL) &&
       (IsTaintImage(image) == MagickFalse))
@@ -1069,7 +1074,6 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
             write_info->magick,&image->exception);
           write_info=DestroyImageInfo(write_info);
           (void) CopyMagickString(image->filename,filename,MaxTextExtent);
-          sans_exception=DestroyExceptionInfo(sans_exception);
           return(status);
         }
     }
@@ -1099,18 +1103,20 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
           (void) CloseBlob(image);
         }
     }
+  if (constitute_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&constitute_semaphore);
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetImageEncoder(magick_info) != (EncodeImageHandler *) NULL))
     {
       /*
         Call appropriate image writer based on image type.
       */
-      thread_support=GetMagickThreadSupport(magick_info);      
+      thread_support=GetMagickThreadSupport(magick_info);
       if ((thread_support & EncoderThreadSupport) == 0)
-        AcquireSemaphoreInfo(&constitute_semaphore);
+        LockSemaphoreInfo(constitute_semaphore);
       status=GetImageEncoder(magick_info)(write_info,image);
       if ((thread_support & EncoderThreadSupport) == 0)
-        RelinquishSemaphoreInfo(constitute_semaphore);
+        UnlockSemaphoreInfo(constitute_semaphore);
     }
   else
     {
@@ -1123,16 +1129,18 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
           */
           *write_info->filename='\0';
           if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
-            AcquireSemaphoreInfo(&constitute_semaphore);
+            LockSemaphoreInfo(constitute_semaphore);
           status=InvokeDelegate(write_info,image,(char *) NULL,
             write_info->magick,&image->exception);
           if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
-            RelinquishSemaphoreInfo(constitute_semaphore);
+            UnlockSemaphoreInfo(constitute_semaphore);
           (void) CopyMagickString(image->filename,filename,MaxTextExtent);
         }
       else
         {
+          sans_exception=AcquireExceptionInfo();
           magick_info=GetMagickInfo(write_info->magick,sans_exception);
+          sans_exception=DestroyExceptionInfo(sans_exception);
           if ((write_info->affirm == MagickFalse) &&
               (magick_info == (const MagickInfo *) NULL))
             {
@@ -1140,6 +1148,12 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
                 MaxTextExtent);
               magick_info=GetMagickInfo(write_info->magick,&image->exception);
             }
+          if ((magick_info == (const MagickInfo *) NULL) ||
+              (GetImageEncoder(magick_info) == (EncodeImageHandler *) NULL))
+            {
+              magick_info=GetMagickInfo(image->magick,&image->exception);
+              (void) CopyMagickString(image->filename,filename,MaxTextExtent);
+            }
           if ((magick_info == (const MagickInfo *) NULL) ||
               (GetImageEncoder(magick_info) == (EncodeImageHandler *) NULL))
             (void) ThrowMagickException(&image->exception,GetMagickModule(),
@@ -1150,12 +1164,12 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
               /*
                 Call appropriate image writer based on image type.
               */
-              thread_support=GetMagickThreadSupport(magick_info);      
+              thread_support=GetMagickThreadSupport(magick_info);
               if ((thread_support & EncoderThreadSupport) == 0)
-                AcquireSemaphoreInfo(&constitute_semaphore);
+                LockSemaphoreInfo(constitute_semaphore);
               status=GetImageEncoder(magick_info)(write_info,image);
               if ((thread_support & EncoderThreadSupport) == 0)
-                RelinquishSemaphoreInfo(constitute_semaphore);
+                UnlockSemaphoreInfo(constitute_semaphore);
             }
         }
     }
@@ -1179,7 +1193,6 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
       (write_info->verbose != MagickFalse))
     (void) IdentifyImage(image,stdout,MagickFalse);
   write_info=DestroyImageInfo(write_info);
-  sans_exception=DestroyExceptionInfo(sans_exception);
   return(status);
 }
 
@@ -1215,6 +1228,8 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
 MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
   Image *images,const char *filename,ExceptionInfo *exception)
 {
+#define WriteImageTag  "Write/Image"
+
   BlobInfo
     *blob;
 
@@ -1224,6 +1239,18 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
   ImageInfo
     *write_info;
 
+  MagickBooleanType
+    proceed;
+
+  MagickOffsetType
+    i;
+
+  MagickProgressMonitor
+    progress_monitor;
+
+  MagickSizeType
+    number_images;
+
   MagickStatusType
     status;
 
@@ -1249,7 +1276,8 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
   if (*write_info->magick == '\0')
     (void) CopyMagickString(write_info->magick,images->magick,MaxTextExtent);
   sans_exception=AcquireExceptionInfo();
-  (void) SetImageInfo(write_info,MagickTrue,sans_exception);
+  (void) SetImageInfo(write_info,(unsigned int) GetImageListLength(images),
+    sans_exception);
   sans_exception=DestroyExceptionInfo(sans_exception);
   p=images;
   for ( ; GetNextImageInList(p) != (Image *) NULL; p=GetNextImageInList(p))
@@ -1270,12 +1298,26 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
     Write images.
   */
   status=MagickTrue;
+  progress_monitor=(MagickProgressMonitor) NULL;
+  i=0;
+  number_images=GetImageListLength(images);
   for (p=images; p != (Image *) NULL; p=GetNextImageInList(p))
   {
+    if (number_images != 1)
+      progress_monitor=SetImageProgressMonitor(p,(MagickProgressMonitor) NULL,
+        p->client_data);
     status&=WriteImage(write_info,p);
     GetImageException(p,exception);
+    if (number_images != 1)
+      (void) SetImageProgressMonitor(p,progress_monitor,p->client_data);
     if (write_info->adjoin != MagickFalse)
       break;
+    if (number_images != 1)
+      {
+        proceed=SetImageProgress(p,WriteImageTag,i++,number_images);
+        if (proceed == MagickFalse)
+          break;
+      }
   }
   write_info=DestroyImageInfo(write_info);
   return(status != 0 ? MagickTrue : MagickFalse);
