@@ -17,7 +17,7 @@
 %                              January 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,6 +47,7 @@
 #include "magick/geometry.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/token.h"
 
 /*
@@ -655,12 +656,12 @@ MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
       GetMagickToken(p,&p,token);
     switch (i)
     {
-      case 0: affine_matrix->sx=atof(token); break;
-      case 1: affine_matrix->rx=atof(token); break;
-      case 2: affine_matrix->ry=atof(token); break;
-      case 3: affine_matrix->sy=atof(token); break;
-      case 4: affine_matrix->tx=atof(token); flags|=XValue; break;
-      case 5: affine_matrix->ty=atof(token); flags|=YValue; break;
+      case 0: affine_matrix->sx=StringToDouble(token); break;
+      case 1: affine_matrix->rx=StringToDouble(token); break;
+      case 2: affine_matrix->ry=StringToDouble(token); break;
+      case 3: affine_matrix->sy=StringToDouble(token); break;
+      case 4: affine_matrix->tx=StringToDouble(token); flags|=XValue; break;
+      case 5: affine_matrix->ty=StringToDouble(token); flags|=YValue; break;
     }
   }
   determinant=(affine_matrix->sx*affine_matrix->sy-affine_matrix->rx*
@@ -863,7 +864,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       /*
         Parse xi value.
       */
-      if ((*p == ',') || (*p == '/'))
+      if ((*p == ',') || (*p == '/') || (*p == ':'))
         p++;
       q=p;
       value=strtod(p,&p);
@@ -882,7 +883,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
           /*
             Parse psi value.
           */
-          if ((*p == ',') || (*p == '/'))
+          if ((*p == ',') || (*p == '/') || (*p == ':'))
             p++;
           q=p;
           value=strtod(p,&p);
@@ -902,7 +903,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
           /*
             Parse chi value.
           */
-          if ((*p == ',') || (*p == '/'))
+          if ((*p == ',') || (*p == '/') || (*p == ':'))
             p++;
           q=p;
           value=strtod(p,&p);
@@ -925,6 +926,17 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       if (geometry_info->xi == 0.0)
         geometry_info->sigma=2.0;
     }
+  if (((flags & SigmaValue) == 0) && ((flags & XiValue) != 0) &&
+      ((flags & PsiValue) == 0))
+    {
+      /*
+        Support negative height values (e.g. 30x-20).
+      */
+      geometry_info->sigma=geometry_info->xi;
+      geometry_info->xi=0.0;
+      flags|=SigmaValue;
+      flags&=(~XiValue);
+    }
   return(flags);
 }
 
@@ -1002,12 +1014,13 @@ MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
       scale.y=geometry_info.sigma;
       if ((status & SigmaValue) == 0)
         scale.y=scale.x;
-      region_info->width=(unsigned long) ((scale.x*image->columns/100.0)+0.5);
-      region_info->height=(unsigned long) ((scale.y*image->rows/100.0)+0.5);
+      region_info->width=(unsigned long) floor((scale.x*image->columns/100.0)+
+        0.5);
+      region_info->height=(unsigned long) floor((scale.y*image->rows/100.0)+
+        0.5);
     }
   /*
-    If a region width of 0 is given by the user, use the images virtual canvas
-    size for the gravity offset adjustments-- restore afterward.
+    Adjust offset according to gravity setting.
   */
   width=region_info->width;
   height=region_info->height;
@@ -1108,10 +1121,10 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
       scale.y=geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
         scale.y=scale.x;
-      *width=(unsigned long) (scale.x*former_width/100.0+0.5);
+      *width=(unsigned long) floor(scale.x*former_width/100.0+0.5);
       if (*width == 0)
         *width=1;
-      *height=(unsigned long) (scale.y*former_height/100.0+0.5);
+      *height=(unsigned long) floor(scale.y*former_height/100.0+0.5);
       if (*height == 0)
         *height=1;
       former_width=(*width);
@@ -1166,8 +1179,10 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
                 scale_factor=(MagickRealType) *height/(MagickRealType)
                   former_width;
             }
-      *width=MagickMax((unsigned long) (scale_factor*former_width+0.5),1UL);
-      *height=MagickMax((unsigned long) (scale_factor*former_height+0.5),1UL);
+      *width=MagickMax((unsigned long) floor(scale_factor*former_width+0.5),
+        1UL);
+      *height=MagickMax((unsigned long) floor(scale_factor*former_height+0.5),
+        1UL);
     }
   if ((flags & GreaterValue) != 0)
     {
@@ -1275,18 +1290,18 @@ MagickExport MagickStatusType ParsePageGeometry(const Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   P a r s e S i z e G e o m e t r y                                         %
+%   P a r s e R e g i o n G e o m e t r y                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ParseSizeGeometry() returns a region as defined by the geometry string with
-%  respect to the image dimensions and aspect ratio.
+%  ParseRegionGeometry() returns a region as defined by the geometry string
+%  with respect to the image dimensions and aspect ratio.
 %
-%  The format of the ParseSizeGeometry method is:
+%  The format of the ParseRegionGeometry method is:
 %
-%      MagickStatusType ParseSizeGeometry(const Image *image,
+%      MagickStatusType ParseRegionGeometry(const Image *image,
 %        const char *geometry,RectangeInfo *region_info,
 %        ExceptionInfo *exception)
 %
@@ -1299,7 +1314,7 @@ MagickExport MagickStatusType ParsePageGeometry(const Image *image,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport MagickStatusType ParseSizeGeometry(const Image *image,
+MagickExport MagickStatusType ParseRegionGeometry(const Image *image,
   const char *geometry,RectangleInfo *region_info,ExceptionInfo *exception)
 {
   MagickStatusType
