@@ -18,7 +18,7 @@
 %                             November 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -65,10 +65,14 @@
 #include "magick/random_.h"
 #include "magick/registry.h"
 #include "magick/resource_.h"
+#include "magick/policy.h"
 #include "magick/semaphore.h"
 #include "magick/signature-private.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
+#include "magick/thread_.h"
+#include "magick/thread-private.h"
 #include "magick/token.h"
 #include "magick/utility.h"
 #include "magick/xwindow-private.h"
@@ -121,40 +125,6 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   D e s t r o y M a g i c k L i s t                                         %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  DestroyMagickList() deallocates memory associated with the MagickInfo list.
-%
-%  The format of the DestroyMagickList method is:
-%
-%      void DestroyMagickList(void)
-%
-*/
-MagickExport void DestroyMagickList(void)
-{
-  AcquireSemaphoreInfo(&magick_semaphore);
-  if (magick_list != (SplayTreeInfo *) NULL)
-    magick_list=DestroySplayTree(magick_list);
-  instantiate_magick=MagickFalse;
-  RelinquishSemaphoreInfo(magick_semaphore);
-  DestroySemaphoreInfo(&magick_semaphore);
-#if !defined(MAGICKCORE_BUILD_MODULES)
-  UnregisterStaticModules();
-#endif
-#if defined(MAGICKCORE_MODULES_SUPPORT)
-  DestroyModuleList();
-#endif
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 +   G e t I m a g e D e c o d e r                                             %
 %                                                                             %
 %                                                                             %
@@ -222,12 +192,13 @@ MagickExport EncodeImageHandler *GetImageEncoder(const MagickInfo *magick_info)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  GetImageMagick() searches for an image format that matches the specified
-%  magick string.  If one is found the name is returned otherwise NULL.
+%  magick string.  If one is found, MagickTrue is returned otherwise
+%  MagickFalse.
 %
 %  The format of the GetImageMagick method is:
 %
-%      const char *GetImageMagick(const unsigned char *magick,
-%        const size_t length)
+%      MagickBooleanType GetImageMagick(const unsigned char *magick,
+%        const size_t length,char *format)
 %
 %  A description of each parameter follows:
 %
@@ -235,13 +206,18 @@ MagickExport EncodeImageHandler *GetImageEncoder(const MagickInfo *magick_info)
 %
 %    o length: the length of the binary string.
 %
+%    o format: the image format as determined by the magick bytes.
+%
 */
-MagickExport const char *GetImageMagick(const unsigned char *magick,
-  const size_t length)
+MagickExport MagickBooleanType GetImageMagick(const unsigned char *magick,
+  const size_t length,char *format)
 {
   ExceptionInfo
     *exception;
 
+  MagickBooleanType
+    status;
+
   register const MagickInfo
     *p;
 
@@ -251,21 +227,24 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
   p=GetMagickInfo("*",exception);
   exception=DestroyExceptionInfo(exception);
   if (p == (const MagickInfo *) NULL)
-    return((const char *) NULL);
-  AcquireSemaphoreInfo(&magick_semaphore);
+    return(MagickFalse);
+  status=MagickFalse;
+  LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   while (p != (const MagickInfo *) NULL)
   {
     if ((p->magick != (IsImageFormatHandler *) NULL) &&
         (p->magick(magick,length) != 0))
-      break;
+      {
+        status=MagickTrue;
+        (void) CopyMagickString(format,p->name,MaxTextExtent);
+        break;
+      }
     p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   }
-  RelinquishSemaphoreInfo(magick_semaphore);
-  if (p != (MagickInfo *) NULL)
-    return(p->name);
-  return((const char *) NULL);
+  UnlockSemaphoreInfo(magick_semaphore);
+  return(status);
 }
 
 /*
@@ -371,7 +350,7 @@ MagickExport const char *GetMagickDescription(const MagickInfo *magick_info)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  GetMagickEndianSupport() returns the MagickTrue if the coder respects
-%  endianess other than MSBEndian.
+%  endianness other than MSBEndian.
 %
 %  The format of the GetMagickEndianSupport method is:
 %
@@ -434,16 +413,16 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
       if (LocaleCompare(name,"*") == 0)
         (void) OpenModules(exception);
 #endif
-      AcquireSemaphoreInfo(&magick_semaphore);
+      LockSemaphoreInfo(magick_semaphore);
       ResetSplayTreeIterator(magick_list);
-      p=GetNextValueInSplayTree(magick_list);
-      RelinquishSemaphoreInfo(magick_semaphore);
+      p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
+      UnlockSemaphoreInfo(magick_semaphore);
       return(p);
     }
   /*
     Find name in list.
   */
-  AcquireSemaphoreInfo(&magick_semaphore);
+  LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   while (p != (const MagickInfo *) NULL)
@@ -467,7 +446,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
       }
     }
 #endif
-  RelinquishSemaphoreInfo(magick_semaphore);
+  UnlockSemaphoreInfo(magick_semaphore);
   return(p);
 }
 
@@ -548,7 +527,7 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
   /*
     Generate magick list.
   */
-  AcquireSemaphoreInfo(&magick_semaphore);
+  LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   for (i=0; p != (const MagickInfo *) NULL; )
@@ -558,7 +537,7 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
       formats[i++]=p;
     p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   }
-  RelinquishSemaphoreInfo(magick_semaphore);
+  UnlockSemaphoreInfo(magick_semaphore);
   qsort((void *) formats,(size_t) i,sizeof(*formats),MagickInfoCompare);
   formats[i]=(MagickInfo *) NULL;
   *number_formats=(unsigned long) i;
@@ -638,7 +617,7 @@ MagickExport char **GetMagickList(const char *pattern,
     GetNumberOfNodesInSplayTree(magick_list)+1UL,sizeof(*formats));
   if (formats == (char **) NULL)
     return((char **) NULL);
-  AcquireSemaphoreInfo(&magick_semaphore);
+  LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   for (i=0; p != (const MagickInfo *) NULL; )
@@ -648,7 +627,7 @@ MagickExport char **GetMagickList(const char *pattern,
       formats[i++]=ConstantString(p->name);
     p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   }
-  RelinquishSemaphoreInfo(magick_semaphore);
+  UnlockSemaphoreInfo(magick_semaphore);
   qsort((void *) formats,(size_t) i,sizeof(*formats),MagickCompare);
   formats[i]=(char *) NULL;
   *number_formats=(unsigned long) i;
@@ -660,7 +639,49 @@ MagickExport char **GetMagickList(const char *pattern,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   G e t M a g i c k E n d i a n S u p p o r t                               %
+%   G e t M a g i c k P r e c i s i o n                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMagickPrecision() returns the maximum number of significant digits to be
+%  printed.
+%
+%  The format of the GetMagickPrecision method is:
+%
+%      int GetMagickPrecision(void)
+%
+*/
+MagickExport int GetMagickPrecision(void)
+{
+#define MagickPrecision  6
+
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  if (SetMagickPrecision(0) == 0)
+    {
+      char
+        *limit;
+
+      (void) SetMagickPrecision(MagickPrecision);
+      limit=GetEnvironmentValue("MAGICK_PRECISION");
+      if (limit == (char *) NULL)
+        limit=GetPolicyValue("precision");
+      if (limit != (char *) NULL)
+        {
+          (void) SetMagickPrecision(StringToInteger(limit));
+          limit=DestroyString(limit);
+        }
+    }
+  return(SetMagickPrecision(0));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   G e t M a g i c k R a w S u p p o r t                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -793,10 +814,13 @@ static void *DestroyMagickNode(void *magick_info)
 
 static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
 {
+  (void) exception;
   if ((magick_list == (SplayTreeInfo *) NULL) &&
       (instantiate_magick == MagickFalse))
     {
-      AcquireSemaphoreInfo(&magick_semaphore);
+      if (magick_semaphore == (SemaphoreInfo *) NULL)
+        AcquireSemaphoreInfo(&magick_semaphore);
+      LockSemaphoreInfo(magick_semaphore);
       if ((magick_list == (SplayTreeInfo *) NULL) &&
           (instantiate_magick == MagickFalse))
         {
@@ -811,7 +835,7 @@ static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
           if (magick_list == (SplayTreeInfo *) NULL)
             ThrowFatalException(ResourceLimitFatalError,
               "MemoryAllocationFailed");
-          magick_info=SetMagickInfo("tmp");
+          magick_info=SetMagickInfo("ephemeral");
           magick_info->stealth=MagickTrue;
           status=AddValueToSplayTree(magick_list,magick_info->name,magick_info);
           if (status == MagickFalse)
@@ -838,7 +862,7 @@ static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
 #endif
           instantiate_magick=MagickTrue;
         }
-      RelinquishSemaphoreInfo(magick_semaphore);
+      UnlockSemaphoreInfo(magick_semaphore);
     }
   return(magick_list != (SplayTreeInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -854,8 +878,8 @@ static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsMagickConflict() returns MagickTrue if the image format is not a valid
-%  image format or conflicts with a logical drive (.e.g. X:).
+%  IsMagickConflict() returns MagickTrue if the image format conflicts with a
+%  logical drive (.e.g. X:).
 %
 %  The format of the IsMagickConflict method is:
 %
@@ -868,35 +892,16 @@ static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
 */
 MagickExport MagickBooleanType IsMagickConflict(const char *magick)
 {
-  const DelegateInfo
-    *delegate_info;
-
-  const MagickInfo
-    *magick_info;
-
-  ExceptionInfo
-    *exception;
-
   assert(magick != (char *) NULL);
-  exception=AcquireExceptionInfo();
-  magick_info=GetMagickInfo(magick,exception);
-  delegate_info=GetDelegateInfo(magick,(char *) NULL,exception);
-  if (delegate_info == (const DelegateInfo *) NULL)
-    delegate_info=GetDelegateInfo((char *) NULL,magick,exception);
-  exception=DestroyExceptionInfo(exception);
-  if ((magick_info == (const MagickInfo *) NULL) &&
-      (delegate_info == (const DelegateInfo *) NULL))
-    return(MagickTrue);
 #if defined(macintosh)
   return(MACIsMagickConflict(magick));
-#endif
-#if defined(vms)
+#elif defined(vms)
   return(VMSIsMagickConflict(magick));
-#endif
-#if defined(__WINDOWS__)
+#elif defined(__WINDOWS__)
   return(NTIsMagickConflict(magick));
-#endif
+#else
   return(MagickFalse);
+#endif
 }
 
 /*
@@ -996,10 +1001,13 @@ MagickExport MagickBooleanType ListMagickInfo(FILE *file,
           }
       }
   }
-  (void) fprintf(file,"\n* native blob support\n\n");
+  (void) fprintf(file,"\n* native blob support\n");
+  (void) fprintf(file,"r read support\n");
+  (void) fprintf(file,"w write support\n");
+  (void) fprintf(file,"+ support for multiple images\n");
   (void) fflush(file);
-  magick_info=(const MagickInfo **)
-    RelinquishMagickMemory((void *) magick_info);
+  magick_info=(const MagickInfo **) RelinquishMagickMemory((void *)
+    magick_info);
   return(MagickTrue);
 }
 
@@ -1033,6 +1041,60 @@ MagickExport MagickBooleanType IsMagickInstantiated(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   M a g i c k C o m p o n e n t G e n e s i s                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickComponentGenesis() instantiates the magick component.
+%
+%  The format of the MagickComponentGenesis method is:
+%
+%      MagickBooleanType MagickComponentGenesis(void)
+%
+*/
+MagickExport MagickBooleanType MagickComponentGenesis(void)
+{
+  AcquireSemaphoreInfo(&magick_semaphore);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   M a g i c k C o m p o n e n t T e r m i n u s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickComponentTerminus() destroys the magick component.
+%
+%  The format of the MagickComponentTerminus method is:
+%
+%      void MagickComponentTerminus(void)
+%
+*/
+MagickExport void MagickComponentTerminus(void)
+{
+  if (magick_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&magick_semaphore);
+  LockSemaphoreInfo(magick_semaphore);
+  if (magick_list != (SplayTreeInfo *) NULL)
+    magick_list=DestroySplayTree(magick_list);
+  instantiate_magick=MagickFalse;
+  UnlockSemaphoreInfo(magick_semaphore);
+  DestroySemaphoreInfo(&magick_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k C o r e G e n e s i s                                         %
 %                                                                             %
 %                                                                             %
@@ -1041,7 +1103,7 @@ MagickExport MagickBooleanType IsMagickInstantiated(void)
 %
 %  MagickCoreGenesis() initializes the MagickCore environment.
 %
-%  The format of the InitializeMagick function is:
+%  The format of the MagickCoreGenesis function is:
 %
 %      MagickCoreGenesis(const char *path,
 %        const MagickBooleanType establish_signal_handlers)
@@ -1093,7 +1155,7 @@ static void MagickSignalHandler(int signal_number)
 #if !defined(MAGICKCORE_HAVE_SIGACTION)
   (void) signal(signal_number,SIG_IGN);
 #endif
-  AsynchronousDestroyMagickResources();
+  AsynchronousResourceComponentTerminus();
   instantiate_magick=MagickFalse;
   (void) SetMagickSignalHandler(signal_number,signal_handlers[signal_number]);
 #if defined(MAGICKCORE_HAVE_RAISE)
@@ -1143,18 +1205,13 @@ MagickExport void MagickCoreGenesis(const char *path,
     execution_path[MaxTextExtent],
     filename[MaxTextExtent];
 
-  ExceptionInfo
-    *exception;
-
-  time_t
-    seconds;
-
   /*
     Initialize the Magick environment.
   */
-  seconds=time((time_t *) NULL);
-  srand((unsigned int) (seconds+getpid()));
-  InitializeSemaphore();
+  (void) SemaphoreComponentGenesis();
+  (void) LogComponentGenesis();
+  (void) LocaleComponentGenesis();
+  (void) RandomComponentGenesis();
   events=GetEnvironmentValue("MAGICK_DEBUG");
   if (events != (char *) NULL)
     {
@@ -1228,12 +1285,25 @@ MagickExport void MagickCoreGenesis(const char *path,
 #endif
     }
   /*
-    Initialize magick resources.
+    Instantiate magick resources.
   */
-  InitializeMagickResources();
-  exception=AcquireExceptionInfo();
-  (void) GetMagickInfo((char *) NULL,exception);
-  exception=DestroyExceptionInfo(exception);
+  (void) ConfigureComponentGenesis();
+  (void) PolicyComponentGenesis();
+  (void) CacheComponentGenesis();
+  (void) RegistryComponentGenesis();
+  (void) ResourceComponentGenesis();
+  (void) CoderComponentGenesis();
+  (void) MagickComponentGenesis();
+#if defined(MAGICKCORE_MODULES_SUPPORT)
+  (void) ModuleComponentGenesis();
+#endif
+  (void) DelegateComponentGenesis();
+  (void) MagicComponentGenesis();
+  (void) ColorComponentGenesis();
+  (void) TypeComponentGenesis();
+  (void) MimeComponentGenesis();
+  (void) ConstituteComponentGenesis();
+  (void) XComponentGenesis();
 }
 
 /*
@@ -1249,7 +1319,7 @@ MagickExport void MagickCoreGenesis(const char *path,
 %
 %  MagickCoreTerminus() destroys the MagickCore environment.
 %
-%  The format of the DestroyMagick function is:
+%  The format of the MagickCoreTerminus function is:
 %
 %      MagickCoreTerminus(void)
 %
@@ -1257,26 +1327,34 @@ MagickExport void MagickCoreGenesis(const char *path,
 MagickExport void MagickCoreTerminus(void)
 {
 #if defined(MAGICKCORE_X11_DELEGATE)
-  DestroyXResources();
+  XComponentTerminus();
 #endif
-  DestroyConstitute();
-  DestroyMimeList();
-  DestroyConfigureList();
-  DestroyTypeList();
-  DestroyColorList();
+  ConstituteComponentTerminus();
+  MimeComponentTerminus();
+  TypeComponentTerminus();
+  ColorComponentTerminus();
 #if defined(__WINDOWS__)
   NTGhostscriptUnLoadDLL();
 #endif
-  DestroyMagicList();
-  DestroyDelegateList();
-  DestroyMagickList();
-  DestroyCoderList();
-  DestroyMagickResources();
-  DestroyImageRegistry();
-  DestroyPixelCacheResources();
-  DestroyRandomReservoir();
-  DestroyLocaleList();
-  DestroyLogList();
+  MagicComponentTerminus();
+  DelegateComponentTerminus();
+  MagickComponentTerminus();
+#if !defined(MAGICKCORE_BUILD_MODULES)
+  UnregisterStaticModules();
+#endif
+#if defined(MAGICKCORE_MODULES_SUPPORT)
+  ModuleComponentTerminus();
+#endif
+  CoderComponentTerminus();
+  ResourceComponentTerminus();
+  RegistryComponentTerminus();
+  CacheComponentTerminus();
+  PolicyComponentTerminus();
+  ConfigureComponentTerminus();
+  RandomComponentTerminus();
+  LocaleComponentTerminus();
+  LogComponentTerminus();
+  SemaphoreComponentTerminus();
   instantiate_magick=MagickFalse;
 }
 
@@ -1359,7 +1437,7 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
 
   assert(name != (const char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",name);
-  magick_info=(MagickInfo *) AcquireMagickMemory(sizeof(*magick_info));
+  magick_info=(MagickInfo *) AcquireAlignedMemory(1,sizeof(*magick_info));
   if (magick_info == (MagickInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   (void) ResetMagickMemory(magick_info,0,sizeof(*magick_info));
@@ -1377,6 +1455,40 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S e t M a g i c k P r e c i s i o n                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetMagickPrecision() sets the maximum number of significant digits to be
+%  printed and returns it.
+%
+%  The format of the SetMagickPrecision method is:
+%
+%      int SetMagickPrecision(const int precision)
+%
+%  A description of each parameter follows:
+%
+%    o precision: set the maximum number of significant digits to be printed.
+%
+*/
+MagickExport int SetMagickPrecision(const int precision)
+{
+  static int
+    magick_precision = 0;
+
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  if (precision != 0)
+    magick_precision=precision;
+  return(magick_precision);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   U n r e g i s t e r M a g i c k I n f o                                   %
 %                                                                             %
 %                                                                             %
@@ -1409,7 +1521,7 @@ MagickExport MagickBooleanType UnregisterMagickInfo(const char *name)
     return(MagickFalse);
   if (GetNumberOfNodesInSplayTree(magick_list) == 0)
     return(MagickFalse);
-  AcquireSemaphoreInfo(&magick_semaphore);
+  LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   while (p != (const MagickInfo *) NULL)
@@ -1419,6 +1531,6 @@ MagickExport MagickBooleanType UnregisterMagickInfo(const char *name)
     p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   }
   status=DeleteNodeByValueFromSplayTree(magick_list,p);
-  RelinquishSemaphoreInfo(magick_semaphore);
+  UnlockSemaphoreInfo(magick_semaphore);
   return(status);
 }
