@@ -17,7 +17,7 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -56,6 +56,7 @@
 #include "magick/semaphore.h"
 #include "magick/signature-private.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/splay-tree.h"
 #include "magick/thread-private.h"
 #include "magick/token.h"
@@ -158,10 +159,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     limit;
 
   status=MagickFalse;
-  (void) FormatMagickSize(size,resource_request);
+  (void) FormatMagickSize(size,MagickFalse,resource_request);
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     AcquireSemaphoreInfo(&resource_semaphore);
-  (void) LockSemaphoreInfo(resource_semaphore);
+  LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -170,9 +171,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       limit=resource_info.area_limit;
       status=(resource_info.area_limit == MagickResourceInfinity) ||
         (size < limit) ? MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.area,
+      (void) FormatMagickSize((MagickSizeType) resource_info.area,MagickFalse,
         resource_current);
-      (void) FormatMagickSize(resource_info.area_limit,resource_limit);
+      (void) FormatMagickSize(resource_info.area_limit,MagickFalse,
+        resource_limit);
       break;
     }
     case MemoryResource:
@@ -182,9 +184,9 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.memory_limit == MagickResourceInfinity) ||
         ((MagickSizeType) resource_info.memory < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.memory,
+      (void) FormatMagickSize((MagickSizeType) resource_info.memory,MagickTrue,
         resource_current);
-      (void) FormatMagickSize(resource_info.memory_limit,
+      (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,
         resource_limit);
       break;
     }
@@ -195,9 +197,9 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.map_limit == MagickResourceInfinity) ||
         ((MagickSizeType) resource_info.map < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.map,
+      (void) FormatMagickSize((MagickSizeType) resource_info.map,MagickTrue,
         resource_current);
-      (void) FormatMagickSize(resource_info.map_limit,
+      (void) FormatMagickSize(resource_info.map_limit,MagickTrue,
         resource_limit);
       break;
     }
@@ -208,9 +210,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.disk_limit == MagickResourceInfinity) ||
         ((MagickSizeType) resource_info.disk < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.disk,
+      (void) FormatMagickSize((MagickSizeType) resource_info.disk,MagickFalse,
         resource_current);
-      (void) FormatMagickSize(resource_info.disk_limit,resource_limit);
+      (void) FormatMagickSize(resource_info.disk_limit,MagickFalse,
+        resource_limit);
       break;
     }
     case FileResource:
@@ -220,10 +223,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.file_limit == MagickResourceInfinity) ||
         ((MagickSizeType) resource_info.file < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.file,
+      (void) FormatMagickSize((MagickSizeType) resource_info.file,MagickFalse,
         resource_current);
       (void) FormatMagickSize((MagickSizeType) resource_info.file_limit,
-        resource_limit);
+        MagickFalse,resource_limit);
       break;
     }
     case ThreadResource:
@@ -233,10 +236,10 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.thread_limit == MagickResourceInfinity) ||
         ((MagickSizeType) resource_info.thread < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.thread,
+      (void) FormatMagickSize((MagickSizeType) resource_info.thread,MagickFalse,
         resource_current);
       (void) FormatMagickSize((MagickSizeType) resource_info.thread_limit,
-        resource_limit);
+        MagickFalse,resource_limit);
       break;
     }
     case TimeResource:
@@ -246,17 +249,17 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.time_limit == MagickResourceInfinity) ||
         ((MagickSizeType) resource_info.time < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.time,
+      (void) FormatMagickSize((MagickSizeType) resource_info.time,MagickFalse,
         resource_current);
       (void) FormatMagickSize((MagickSizeType) resource_info.time_limit,
-        resource_limit);
+        MagickFalse,resource_limit);
       break;
     }
     default:
       break;
   }
-  (void) UnlockSemaphoreInfo(resource_semaphore);
-  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
+  UnlockSemaphoreInfo(resource_semaphore);
+  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %sB/%sB/%sB",
     MagickOptionToMnemonic(MagickResourceOptions,(long) type),resource_request,
     resource_current,resource_limit);
   return(status);
@@ -471,11 +474,11 @@ MagickExport int AcquireUniqueFileResource(char *path)
     return(file);
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     AcquireSemaphoreInfo(&resource_semaphore);
-  (void) LockSemaphoreInfo(resource_semaphore);
+  LockSemaphoreInfo(resource_semaphore);
   if (temporary_resources == (SplayTreeInfo *) NULL)
     temporary_resources=NewSplayTree(CompareSplayTreeString,
       RelinquishMagickMemory,DestroyTemporaryResources);
-  (void) UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore);
   resource=ConstantString(path);
   (void) AddValueToSplayTree(temporary_resources,resource,resource);
   return(file);
@@ -509,7 +512,7 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
     resource;
 
   resource=0;
-  (void) LockSemaphoreInfo(resource_semaphore);
+  LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -550,7 +553,7 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
     default:
       break;
   }
-  (void) UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore);
   return(resource);
 }
 
@@ -569,7 +572,7 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
 %
 %  The format of the GetMagickResourceLimit() method is:
 %
-%      unsigned long GetMagickResourceLimit(const ResourceType type)
+%      MagickSizeType GetMagickResourceLimit(const ResourceType type)
 %
 %  A description of each parameter follows:
 %
@@ -584,7 +587,7 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
   resource=0;
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     AcquireSemaphoreInfo(&resource_semaphore);
-  (void) LockSemaphoreInfo(resource_semaphore);
+  LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -625,7 +628,7 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
     default:
       break;
   }
-  (void) UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore);
   return(resource);
 }
 
@@ -668,24 +671,24 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
     file=stdout;
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     AcquireSemaphoreInfo(&resource_semaphore);
-  (void) LockSemaphoreInfo(resource_semaphore);
-  (void) FormatMagickSize(resource_info.area_limit,area_limit);
-  (void) FormatMagickSize(resource_info.memory_limit,memory_limit);
-  (void) FormatMagickSize(resource_info.map_limit,map_limit);
-  (void) FormatMagickSize(resource_info.disk_limit,disk_limit);
+  LockSemaphoreInfo(resource_semaphore);
+  (void) FormatMagickSize(resource_info.area_limit,MagickFalse,area_limit);
+  (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,memory_limit);
+  (void) FormatMagickSize(resource_info.map_limit,MagickTrue,map_limit);
+  (void) FormatMagickSize(resource_info.disk_limit,MagickFalse,disk_limit);
   (void) CopyMagickString(time_limit,"unlimited",MaxTextExtent);
   if (resource_info.time_limit != MagickResourceInfinity)
     (void) FormatMagickString(time_limit,MaxTextExtent,"%lu",(unsigned long)
       resource_info.time_limit);
-  (void) fprintf(file,"File        Area      Memory         Map"
-    "        Disk  Thread        Time\n");
-  (void) fprintf(file,"------------------------------------------------------"
-    "------------------\n");
-  (void) fprintf(file,"%4lu  %10s  %10s  %10s  %10s  %6lu  %10s\n",
+  (void) fprintf(file,"File         Area       Memory          Map"
+    "         Disk    Thread         Time\n");
+  (void) fprintf(file,"--------------------------------------------------------"
+    "-----------------------\n");
+  (void) fprintf(file,"%4lu  %10sB  %10sB  %10sB  %10sB    %6lu  %11s\n",
     (unsigned long) resource_info.file_limit,area_limit,memory_limit,map_limit,
     disk_limit,(unsigned long) resource_info.thread_limit,time_limit);
   (void) fflush(file);
-  (void) UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore);
   return(MagickTrue);
 }
 
@@ -722,76 +725,80 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     resource_limit[MaxTextExtent],
     resource_request[MaxTextExtent];
 
-  (void) FormatMagickSize(size,resource_request);
+  (void) FormatMagickSize(size,MagickFalse,resource_request);
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     AcquireSemaphoreInfo(&resource_semaphore);
-  (void) LockSemaphoreInfo(resource_semaphore);
+  LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
     {
       resource_info.area=(MagickOffsetType) size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.area,
+      (void) FormatMagickSize((MagickSizeType) resource_info.area,MagickFalse,
         resource_current);
-      (void) FormatMagickSize(resource_info.area_limit,resource_limit);
+      (void) FormatMagickSize(resource_info.area_limit,MagickFalse,
+        resource_limit);
       break;
     }
     case MemoryResource:
     {
       resource_info.memory-=size;
       (void) FormatMagickSize((MagickSizeType) resource_info.memory,
-        resource_current);
-      (void) FormatMagickSize(resource_info.memory_limit,resource_limit);
+        MagickTrue,resource_current);
+      (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,
+        resource_limit);
       break;
     }
     case MapResource:
     {
       resource_info.map-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.map,
+      (void) FormatMagickSize((MagickSizeType) resource_info.map,MagickTrue,
         resource_current);
-      (void) FormatMagickSize(resource_info.map_limit,resource_limit);
+      (void) FormatMagickSize(resource_info.map_limit,MagickTrue,
+        resource_limit);
       break;
     }
     case DiskResource:
     {
       resource_info.disk-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.disk,
+      (void) FormatMagickSize((MagickSizeType) resource_info.disk,MagickFalse,
         resource_current);
-      (void) FormatMagickSize(resource_info.disk_limit,resource_limit);
+      (void) FormatMagickSize(resource_info.disk_limit,MagickFalse,
+        resource_limit);
       break;
     }
     case FileResource:
     {
       resource_info.file-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.file,
+      (void) FormatMagickSize((MagickSizeType) resource_info.file,MagickFalse,
         resource_current);
       (void) FormatMagickSize((MagickSizeType) resource_info.file_limit,
-        resource_limit);
+        MagickFalse,resource_limit);
       break;
     }
     case ThreadResource:
     {
       resource_info.thread-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.thread,
+      (void) FormatMagickSize((MagickSizeType) resource_info.thread,MagickFalse,
         resource_current);
       (void) FormatMagickSize((MagickSizeType) resource_info.thread_limit,
-        resource_limit);
+        MagickFalse,resource_limit);
       break;
     }
     case TimeResource:
     {
       resource_info.time-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.time,
+      (void) FormatMagickSize((MagickSizeType) resource_info.time,MagickFalse,
         resource_current);
       (void) FormatMagickSize((MagickSizeType) resource_info.time_limit,
-        resource_limit);
+        MagickFalse,resource_limit);
       break;
     }
     default:
       break;
   }
-  (void) UnlockSemaphoreInfo(resource_semaphore);
-  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
+  UnlockSemaphoreInfo(resource_semaphore);
+  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %sB/%sB/%sB",
     MagickOptionToMnemonic(MagickResourceOptions,(long) type),resource_request,
     resource_current,resource_limit);
 }
@@ -880,7 +887,7 @@ static inline MagickSizeType StringToSizeType(const char *string,
   double
     value;
 
-  value=StringToDouble(string,interval);
+  value=SiPrefixToDouble(string,interval);
   if (value >= (double) MagickULLConstant(~0))
     return(MagickULLConstant(~0));
   return((MagickSizeType) value);
@@ -961,7 +968,7 @@ MagickExport MagickBooleanType ResourceComponentGenesis(void)
         resources;
 
       if (getrlimit(RLIMIT_NOFILE,&resources) != -1)
-        files=resources.rlim_cur;
+        files=(long) resources.rlim_cur;
   }
 #endif
 #if defined(MAGICKCORE_HAVE_GETDTABLESIZE) && defined(MAGICKCORE_POSIX_SUPPORT)
@@ -986,7 +993,7 @@ MagickExport MagickBooleanType ResourceComponentGenesis(void)
     limit=GetPolicyValue("thread");
   if (limit != (char *) NULL)
     {
-      SetOpenMPMaximumThreads((unsigned long) atol(limit));
+      SetOpenMPMaximumThreads(StringToUnsignedLong(limit));
       (void) SetMagickResourceLimit(ThreadResource,StringToSizeType(limit,
         100.0));
       limit=DestroyString(limit);
@@ -1024,12 +1031,12 @@ MagickExport void ResourceComponentTerminus(void)
 {
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     AcquireSemaphoreInfo(&resource_semaphore);
-  (void) LockSemaphoreInfo(resource_semaphore);
+  LockSemaphoreInfo(resource_semaphore);
   if (temporary_resources != (SplayTreeInfo *) NULL)
     temporary_resources=DestroySplayTree(temporary_resources);
   if (random_info != (RandomInfo *) NULL)
     random_info=DestroyRandomInfo(random_info);
-  (void) UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore);
   DestroySemaphoreInfo(&resource_semaphore);
 }
 
@@ -1063,7 +1070,7 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
 {
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     AcquireSemaphoreInfo(&resource_semaphore);
-  (void) LockSemaphoreInfo(resource_semaphore);
+  LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -1105,6 +1112,6 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
     default:
       break;
   }
-  (void) UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore);
   return(MagickTrue);
 }
