@@ -17,7 +17,7 @@
 %                                  July 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -72,6 +72,8 @@
 #include "magick/resource_.h"
 #include "magick/shear.h"
 #include "magick/statistic.h"
+#include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/threshold.h"
 #include "magick/transform.h"
 
@@ -149,10 +151,10 @@ MagickExport Image *AffineTransformImage(const Image *image,
 %
 %  The format of the CropToFitImage method is:
 %
-%      Image *CropToFitImage(Image **image,const MagickRealType x_shear,
-%        const MagickRealType x_shear,const MagickRealType width,
-%        const MagickRealType height,const MagickBooleanType rotate,
-%        ExceptionInfo *exception)
+%      MagickBooleanType CropToFitImage(Image **image,
+%        const MagickRealType x_shear,const MagickRealType x_shear,
+%        const MagickRealType width,const MagickRealType height,
+%        const MagickBooleanType rotate,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -163,10 +165,10 @@ MagickExport Image *AffineTransformImage(const Image *image,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-static inline void CropToFitImage(Image **image,const MagickRealType x_shear,
-  const MagickRealType y_shear,const MagickRealType width,
-  const MagickRealType height,const MagickBooleanType rotate,
-  ExceptionInfo *exception)
+static MagickBooleanType CropToFitImage(Image **image,
+  const MagickRealType x_shear,const MagickRealType y_shear,
+  const MagickRealType width,const MagickRealType height,
+  const MagickBooleanType rotate,ExceptionInfo *exception)
 {
   Image
     *crop_image;
@@ -216,20 +218,19 @@ static inline void CropToFitImage(Image **image,const MagickRealType x_shear,
     if (max.y < extent[i].y)
       max.y=extent[i].y;
   }
-  geometry.x=(long) (min.x+0.5);
-  geometry.y=(long) (min.y+0.5);
-  geometry.width=(unsigned long) ((long) (max.x+0.5)-(long) (min.x+0.5));
-  geometry.height=(unsigned long) ((long) (max.y+0.5)-(long) (min.y+0.5));
+  geometry.x=(long) ceil(min.x-0.5);
+  geometry.y=(long) ceil(min.y-0.5);
+  geometry.width=(unsigned long) floor(max.x-min.x+0.5);
+  geometry.height=(unsigned long) floor(max.y-min.y+0.5);
   page=(*image)->page;
   (void) ParseAbsoluteGeometry("0x0+0+0",&(*image)->page);
   crop_image=CropImage(*image,&geometry,exception);
-  (*image)->page=page;
-  if (crop_image != (Image *) NULL)
-    {
-      crop_image->page=page;
-      *image=DestroyImage(*image);
-      *image=crop_image;
-    }
+  if (crop_image == (Image *) NULL)
+    return(MagickFalse);
+  crop_image->page=page;
+  *image=DestroyImage(*image);
+  *image=crop_image;
+  return(MagickTrue);
 }
 
 /*
@@ -368,7 +369,7 @@ static RadonInfo *AcquireRadonInfo(const Image *image,const unsigned long width,
   RadonInfo
     *radon_info;
 
-  radon_info=(RadonInfo *) AcquireMagickMemory(sizeof(*radon_info));
+  radon_info=(RadonInfo *) AcquireAlignedMemory(1,sizeof(*radon_info));
   if (radon_info == (RadonInfo *) NULL)
     return((RadonInfo *) NULL);
   (void) ResetMagickMemory(radon_info,0,sizeof(*radon_info));
@@ -853,22 +854,22 @@ static void GetImageBackgroundColor(Image *image,const long offset,
     {
       if ((x >= offset) && (x < ((long) image->columns-offset)))
         continue;
-      background.red+=QuantumScale*p->red;
-      background.green+=QuantumScale*p->green;
-      background.blue+=QuantumScale*p->blue;
-      background.opacity+=QuantumScale*p->opacity;
+      background.red+=QuantumScale*GetRedPixelComponent(p);
+      background.green+=QuantumScale*GetGreenPixelComponent(p);
+      background.blue+=QuantumScale*GetBluePixelComponent(p);
+      background.opacity+=QuantumScale*GetOpacityPixelComponent(p);
       count++;
       p++;
     }
   }
   image_view=DestroyCacheView(image_view);
-  image->background_color.red=RoundToQuantum((MagickRealType) QuantumRange*
+  image->background_color.red=ClampToQuantum((MagickRealType) QuantumRange*
     background.red/count);
-  image->background_color.green=RoundToQuantum((MagickRealType) QuantumRange*
+  image->background_color.green=ClampToQuantum((MagickRealType) QuantumRange*
     background.green/count);
-  image->background_color.blue=RoundToQuantum((MagickRealType) QuantumRange*
+  image->background_color.blue=ClampToQuantum((MagickRealType) QuantumRange*
     background.blue/count);
-  image->background_color.opacity=RoundToQuantum((MagickRealType) QuantumRange*
+  image->background_color.opacity=ClampToQuantum((MagickRealType) QuantumRange*
     background.opacity/count);
 }
 
@@ -941,8 +942,8 @@ MagickExport Image *DeskewImage(const Image *image,const double threshold,
   (void) SetImageVirtualPixelMethod(clone_image,BackgroundVirtualPixelMethod);
   degrees=RadiansToDegrees(-atan((double) skew/width/8));
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TransformEvent,GetMagickModule(),"  Deskew angle: %g",
-      degrees);
+    (void) LogMagickEvent(TransformEvent,GetMagickModule(),
+      "  Deskew angle: %g",degrees);
   affine_matrix.sx=cos(DegreesToRadians(fmod((double) degrees,360.0)));
   affine_matrix.rx=sin(DegreesToRadians(fmod((double) degrees,360.0)));
   affine_matrix.ry=(-sin(DegreesToRadians(fmod((double) degrees,360.0))));
@@ -959,7 +960,7 @@ MagickExport Image *DeskewImage(const Image *image,const double threshold,
   /*
     Auto-crop image.
   */
-  GetImageBackgroundColor(clone_image,atol(artifact),exception);
+  GetImageBackgroundColor(clone_image,StringToLong(artifact),exception);
   deskew_image=AffineTransformImage(clone_image,&affine_matrix,exception);
   clone_image=DestroyImage(clone_image);
   if (deskew_image == (Image *) NULL)
@@ -1074,7 +1075,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
         Rotate 90 degrees.
       */
       GetPixelCacheTileSize(image,&tile_width,&tile_height);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
+#if defined(MAGICKCORE_OPENMP_SUPPORT) && (_OPENMP > 202001)
   #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
 #endif
       for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height)
@@ -1176,7 +1177,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
               status=MagickFalse;
           }
       }
-      (void) SetImageProgress(image,RotateImageTag,image->rows-1,image->rows);
+      (void) SetImageProgress(image,RotateImageTag,(MagickOffsetType)
+        image->rows-1,image->rows);
       Swap(page.width,page.height);
       Swap(page.x,page.y);
       if (page.width != 0)
@@ -1188,7 +1190,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       /*
         Rotate 180 degrees.
       */
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
+#if defined(MAGICKCORE_OPENMP_SUPPORT) && (_OPENMP > 202001)
   #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
 #endif
       for (y=0; y < (long) image->rows; y++)
@@ -1264,7 +1266,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
         Rotate 270 degrees.
       */
       GetPixelCacheTileSize(image,&tile_width,&tile_height);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
+#if defined(MAGICKCORE_OPENMP_SUPPORT) && (_OPENMP > 202001)
   #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
 #endif
       for (tile_y=0; tile_y < (long) image->rows; tile_y+=tile_height)
@@ -1365,7 +1367,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
               status=MagickFalse;
           }
       }
-      (void) SetImageProgress(image,RotateImageTag,image->rows-1,image->rows);
+      (void) SetImageProgress(image,RotateImageTag,(MagickOffsetType)
+        image->rows-1,image->rows);
       Swap(page.width,page.height);
       Swap(page.x,page.y);
       if (page.height != 0)
@@ -1403,7 +1406,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
 %
 %      MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
 %        const unsigned long width,const unsigned long height,
-%        const long x_offset,const long y_offset)
+%        const long x_offset,const long y_offset,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -1415,10 +1418,12 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
 %    o width, height, x_offset, y_offset: Defines a region of the image
 %      to shear.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
   const unsigned long width,const unsigned long height,const long x_offset,
-  const long y_offset)
+  const long y_offset,ExceptionInfo *exception)
 {
 #define XShearImageTag  "XShear/Image"
 
@@ -1431,9 +1436,6 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
   CacheView
     *image_view;
 
-  ExceptionInfo
-    *exception;
-
   long
     progress,
     y;
@@ -1458,7 +1460,6 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
   */
   status=MagickTrue;
   progress=0;
-  exception=(&image->exception);
   image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
@@ -1623,7 +1624,7 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
 %
 %      MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
 %        const unsigned long width,const unsigned long height,
-%        const long x_offset,const long y_offset)
+%        const long x_offset,const long y_offset,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -1635,10 +1636,12 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
 %    o width, height, x_offset, y_offset: Defines a region of the image
 %      to shear.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
   const unsigned long width,const unsigned long height,const long x_offset,
-  const long y_offset)
+  const long y_offset,ExceptionInfo *exception)
 {
 #define YShearImageTag  "YShear/Image"
 
@@ -1651,9 +1654,6 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
   CacheView
     *image_view;
 
-  ExceptionInfo
-    *exception;
-
   long
     progress,
     x;
@@ -1678,7 +1678,6 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
   */
   status=MagickTrue;
   progress=0;
-  exception=(&image->exception);
   image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
@@ -1872,6 +1871,9 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
     x_offset,
     y_offset;
 
+  MagickBooleanType
+    status;
+
   MagickRealType
     angle;
 
@@ -1930,9 +1932,9 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
       width=image->rows;
       height=image->columns;
     }
-  y_width=width+(long) (fabs(shear.x)*height+0.5);
-  x_offset=(long) (width+((fabs(shear.y)*height)-width)/2.0+0.5);
-  y_offset=(long) (height+((fabs(shear.y)*y_width)-height)/2.0+0.5);
+  y_width=width+(long) floor(fabs(shear.x)*height+0.5);
+  x_offset=(long) ceil(width+((fabs(shear.y)*height)-width)/2.0-0.5);
+  y_offset=(long) ceil(height+((fabs(shear.y)*y_width)-height)/2.0-0.5);
   /*
     Surround image with a border.
   */
@@ -1947,14 +1949,34 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
   /*
     Rotate the image.
   */
-  (void) XShearImage(rotate_image,shear.x,width,height,x_offset,
-    ((long) rotate_image->rows-height)/2);
-  (void) YShearImage(rotate_image,shear.y,y_width,height,
-    ((long) rotate_image->columns-y_width)/2,y_offset);
-  (void) XShearImage(rotate_image,shear.x,y_width,rotate_image->rows,
-    ((long) rotate_image->columns-y_width)/2,0);
-  CropToFitImage(&rotate_image,shear.x,shear.y,(MagickRealType) width,
+  status=XShearImage(rotate_image,shear.x,width,height,x_offset,((long)
+    rotate_image->rows-height)/2,exception);
+  if (status == MagickFalse)
+    {
+      rotate_image=DestroyImage(rotate_image);
+      return((Image *) NULL);
+    }
+  status=YShearImage(rotate_image,shear.y,y_width,height,((long)
+    rotate_image->columns-y_width)/2,y_offset,exception);
+  if (status == MagickFalse)
+    {
+      rotate_image=DestroyImage(rotate_image);
+      return((Image *) NULL);
+    }
+  status=XShearImage(rotate_image,shear.x,y_width,rotate_image->rows,((long)
+    rotate_image->columns-y_width)/2,0,exception);
+  if (status == MagickFalse)
+    {
+      rotate_image=DestroyImage(rotate_image);
+      return((Image *) NULL);
+    }
+  status=CropToFitImage(&rotate_image,shear.x,shear.y,(MagickRealType) width,
     (MagickRealType) height,MagickTrue,exception);
+  if (status == MagickFalse)
+    {
+      rotate_image=DestroyImage(rotate_image);
+      return((Image *) NULL);
+    }
   rotate_image->compose=image->compose;
   rotate_image->page.width=0;
   rotate_image->page.height=0;
@@ -2011,6 +2033,9 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
     x_offset,
     y_offset;
 
+  MagickBooleanType
+    status;
+
   PointInfo
     shear;
 
@@ -2051,10 +2076,11 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   /*
     Compute image size.
   */
-  y_width=image->columns+(long) (fabs(shear.x)*image->rows+0.5);
-  x_offset=(long) (image->columns+((fabs(shear.x)*image->rows)-image->columns)/
-    2.0+0.5);
-  y_offset=(long) (image->rows+((fabs(shear.y)*y_width)-image->rows)/2.0+0.5);
+  y_width=image->columns+(long) floor(fabs(shear.x)*image->rows+0.5);
+  x_offset=(long) ceil(image->columns+((fabs(shear.x)*image->rows)-
+    image->columns)/2.0-0.5);
+  y_offset=(long) ceil(image->rows+((fabs(shear.y)*y_width)-image->rows)/2.0-
+    0.5);
   /*
     Surround image with border.
   */
@@ -2063,20 +2089,35 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   border_info.width=(unsigned long) x_offset;
   border_info.height=(unsigned long) y_offset;
   shear_image=BorderImage(integral_image,&border_info,exception);
+  integral_image=DestroyImage(integral_image);
   if (shear_image == (Image *) NULL)
     ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
-  integral_image=DestroyImage(integral_image);
   /*
     Shear the image.
   */
   if (shear_image->matte == MagickFalse)
     (void) SetImageAlphaChannel(shear_image,OpaqueAlphaChannel);
-  (void) XShearImage(shear_image,shear.x,image->columns,image->rows,x_offset,
-    ((long) shear_image->rows-image->rows)/2);
-  (void) YShearImage(shear_image,shear.y,y_width,image->rows,
-    ((long) shear_image->columns-y_width)/2,y_offset);
-  CropToFitImage(&shear_image,shear.x,shear.y,(MagickRealType) image->columns,
-    (MagickRealType) image->rows,MagickFalse,exception);
+  status=XShearImage(shear_image,shear.x,image->columns,image->rows,x_offset,
+    ((long) shear_image->rows-image->rows)/2,exception);
+  if (status == MagickFalse)
+    {
+      shear_image=DestroyImage(shear_image);
+      return((Image *) NULL);
+    }
+  status=YShearImage(shear_image,shear.y,y_width,image->rows,((long)
+    shear_image->columns-y_width)/2,y_offset,exception);
+  if (status == MagickFalse)
+    {
+      shear_image=DestroyImage(shear_image);
+      return((Image *) NULL);
+    }
+  status=CropToFitImage(&shear_image,shear.x,shear.y,(MagickRealType)
+    image->columns,(MagickRealType) image->rows,MagickFalse,exception);
+  if (status == MagickFalse)
+    {
+      shear_image=DestroyImage(shear_image);
+      return((Image *) NULL);
+    }
   shear_image->compose=image->compose;
   shear_image->page.width=0;
   shear_image->page.height=0;
