@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -26,15 +26,17 @@ extern "C" {
   Define declarations.
 */
 #define MagickPackageName "ImageMagick"
-#define MagickCopyright  "Copyright (C) 1999-2009 ImageMagick Studio LLC"
-#define MagickLibVersion  0x658
-#define MagickLibVersionText  "6.5.8"
-#define MagickLibVersionNumber  2,0,1
-#define MagickLibSubversion  "-2"
-#define MagickReleaseDate  "2009-11-28"
-#define MagickChangeDate   "20091128"
+#define MagickCopyright  "Copyright (C) 1999-2010 ImageMagick Studio LLC"
+#define MagickLibVersion  0x660
+#define MagickLibVersionText  "6.6.0"
+#define MagickLibVersionNumber  3,0,0
+#define MagickLibAddendum  "-8"
+#define MagickLibInterface  3
+#define MagickLibMinInterface  3
+#define MagickReleaseDate  "2010-03-20"
+#define MagickChangeDate   "20100120"
 #define MagickAuthoritativeURL  "http://www.imagemagick.org"
-#define MagickHomeURL  "file:///usr/share/doc/ImageMagick-6.5.8/index.html"
+#define MagickHomeURL  "file:///usr/share/doc/ImageMagick-6.6.0/index.html"
 #if (MAGICKCORE_QUANTUM_DEPTH == 8)
 #define MagickQuantumDepth  "Q8"
 #define MagickQuantumRange  "255"
@@ -69,7 +71,7 @@ extern "C" {
 #define MagickFeatures \
   MagickHDRIFeature MagickOPENMPFeature MagickOPENCLFeature
 #define MagickVersion  \
-  MagickPackageName " " MagickLibVersionText MagickLibSubversion " " \
+  MagickPackageName " " MagickLibVersionText MagickLibAddendum " " \
   MagickReleaseDate " " MagickQuantumDepth " " MagickAuthoritativeURL
 
 extern MagickExport char
