@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -33,7 +33,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%
+%  Use the composite program to overlap one image over another.
 %
 */
 
@@ -43,6 +43,7 @@
 #include "wand/studio.h"
 #include "wand/MagickWand.h"
 #include "wand/mogrify-private.h"
+#include "magick/string-private.h"
 
 /*
   Typedef declarations.
@@ -53,12 +54,8 @@ typedef struct _CompositeOptions
     channel;
 
   char
-    *blend_geometry,
-    *displace_geometry,
-    *dissolve_geometry,
-    *geometry,
-    *unsharp_geometry,
-    *watermark_geometry;
+    *compose_args,
+    *geometry;
 
   CompositeOperator
     compose;
@@ -69,6 +66,9 @@ typedef struct _CompositeOptions
   long
     stegano;
 
+  RectangleInfo
+    offset;
+
   MagickBooleanType
     stereo,
     tile;
@@ -125,21 +125,21 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
   if (composite_image != (Image *) NULL)
     {
       assert(composite_image->signature == MagickSignature);
-      if (composite_options->compose == BlendCompositeOp)
-        (void) CloneString(&composite_image->geometry,
-          composite_options->blend_geometry);
-      if (composite_options->compose == DisplaceCompositeOp)
-        (void) CloneString(&composite_image->geometry,
-          composite_options->displace_geometry);
-      if (composite_options->compose == DissolveCompositeOp)
-        (void) CloneString(&composite_image->geometry,
-          composite_options->dissolve_geometry);
-      if (composite_options->compose == ModulateCompositeOp)
-        (void) CloneString(&composite_image->geometry,
-          composite_options->watermark_geometry);
-      if (composite_options->compose == ThresholdCompositeOp)
-        (void) CloneString(&composite_image->geometry,
-          composite_options->unsharp_geometry);
+      switch( composite_options->compose )
+        {
+          case BlendCompositeOp:
+          case BlurCompositeOp:
+          case DisplaceCompositeOp:
+          case DistortCompositeOp:
+          case DissolveCompositeOp:
+          case ModulateCompositeOp:
+          case ThresholdCompositeOp:
+            (void) SetImageArtifact(composite_image,"compose:args",
+              composite_options->compose_args);
+            break;
+          default:
+            break;
+        }
       /*
         Composite image.
       */
@@ -162,7 +162,9 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
             Image
               *stereo_image;
 
-            stereo_image=StereoImage(*image,composite_image,exception);
+            stereo_image=StereoAnaglyphImage(*image,composite_image,
+              composite_options->offset.x,composite_options->offset.y,
+              exception);
             if (stereo_image != (Image *) NULL)
               {
                 *image=DestroyImageList(*image);
@@ -182,7 +184,7 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
               /*
                 Tile the composite image.
               */
-              (void) SetImageArtifact(composite_image,"modify-outside-overlay",
+              (void) SetImageArtifact(composite_image,"compose:outside-overlay",
                 "false");
               columns=composite_image->columns;
               for (y=0; y < (long) (*image)->rows; y+=composite_image->rows)
@@ -194,23 +196,23 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
             }
           else
             {
-              char
-                composite_geometry[MaxTextExtent];
-
               RectangleInfo
                 geometry;
 
               /*
-                Digitally composite image.
+                Work out gravity Adjustment of Offset
               */
               SetGeometry(*image,&geometry);
               (void) ParseAbsoluteGeometry(composite_options->geometry,
                 &geometry);
-              (void) FormatMagickString(composite_geometry,MaxTextExtent,
-                "%lux%lu%+ld%+ld",composite_image->columns,
-                composite_image->rows,geometry.x,geometry.y);
+              geometry.width=composite_image->columns;
+              geometry.height=composite_image->rows;
+              GravityAdjustGeometry((*image)->columns,(*image)->rows,
+                composite_options->gravity, &geometry);
               (*image)->gravity=(GravityType) composite_options->gravity;
-              (void) ParseGravityGeometry(*image,composite_geometry,&geometry);
+              /*
+                Digitally composite image.
+              */
               status&=CompositeImageChannel(*image,composite_options->channel,
                 composite_options->compose,composite_image,geometry.x,
                 geometry.y);
@@ -220,7 +222,7 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
   return(status != 0 ? MagickTrue : MagickFalse);
 }
 
-static void CompositeUsage(void)
+static MagickBooleanType CompositeUsage(void)
 {
   const char
     **p;
@@ -242,8 +244,9 @@ static void CompositeUsage(void)
       "-bordercolor color   border color",
       "-colors value        preferred number of colors in the image",
       "-decipher filename    convert cipher pixels to plain pixels",
-      "-displace geometry   shift image pixels defined by a displacement map",
+      "-displace geometry   shift lookup according to a relative displacement map",
       "-dissolve value      dissolve the two images a given percent",
+      "-distort geometry    shift lookup according to a absolute distortion map",
       "-encipher filename   convert plain pixels to cipher pixels",
       "-extract geometry    extract area from image",
       "-geometry geometry   location of the composite image",
@@ -258,7 +261,7 @@ static void CompositeUsage(void)
       "-sharpen geometry    sharpen the image",
       "-shave geometry      shave pixels from the image edges",
       "-stegano offset      hide watermark within an image",
-      "-stereo              combine two image to create a stereo anaglyph",
+      "-stereo geometry     combine two image to create a stereo anaglyph",
       "-strip               strip image of all profiles and comments",
       "-thumbnail geometry  create a thumbnail of the image",
       "-transform           affine transform image",
@@ -271,7 +274,8 @@ static void CompositeUsage(void)
     *settings[]=
     {
       "-affine matrix       affine transform matrix",
-      "-alpha option        activate, deactivate, reset, or set the alpha channel",
+      "-alpha option        on, activate, off, deactivate, set, opaque, copy",
+      "                     transparent, extract, background, or shape",
       "-authenticate password",
       "                     decipher image with this password",
       "-blue-primary point  chromaticity blue primary point",
@@ -305,7 +309,7 @@ static void CompositeUsage(void)
       "-quiet               suppress all warning messages",
       "-red-primary point   chromaticity red primary point",
       "-regard-warnings     pay attention to warning messages",
-      "-respect-parenthesis settings remain in effect until parenthesis boundary",
+      "-respect-parentheses settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-scene value         image scene number",
@@ -330,7 +334,8 @@ static void CompositeUsage(void)
 
 
   (void) printf("Version: %s\n",GetMagickVersion((unsigned long *) NULL));
-  (void) printf("Copyright: %s\n\n",GetMagickCopyright());
+  (void) printf("Copyright: %s\n",GetMagickCopyright());
+  (void) printf("Features: %s\n\n",GetMagickFeatures());
   (void) printf("Usage: %s [options ...] image [options ...] composite\n"
     "  [ [options ...] mask ] [options ...] composite\n",
     GetClientName());
@@ -355,7 +360,7 @@ static void CompositeUsage(void)
   (void) printf(
     "image type as the filename suffix (i.e. image.ps).  Specify 'file' as\n");
   (void) printf("'-' for standard input or output.\n");
-  exit(0);
+  return(MagickFalse);
 }
 
 static void GetCompositeOptions(CompositeOptions *composite_options)
@@ -367,24 +372,9 @@ static void GetCompositeOptions(CompositeOptions *composite_options)
 
 static void RelinquishCompositeOptions(CompositeOptions *composite_options)
 {
-  if (composite_options->blend_geometry != (char *) NULL)
-    composite_options->blend_geometry=(char *)
-      RelinquishMagickMemory(composite_options->blend_geometry);
-  if (composite_options->displace_geometry != (char *) NULL)
-    composite_options->displace_geometry=(char *)
-      RelinquishMagickMemory(composite_options->displace_geometry);
-  if (composite_options->dissolve_geometry != (char *) NULL)
-    composite_options->dissolve_geometry=(char *)
-      RelinquishMagickMemory(composite_options->dissolve_geometry);
-  if (composite_options->geometry != (char *) NULL)
-    composite_options->geometry=(char *)
-      RelinquishMagickMemory(composite_options->geometry);
-  if (composite_options->unsharp_geometry != (char *) NULL)
-    composite_options->unsharp_geometry=(char *)
-      RelinquishMagickMemory(composite_options->unsharp_geometry);
-  if (composite_options->watermark_geometry != (char *) NULL)
-    composite_options->watermark_geometry=(char *)
-      RelinquishMagickMemory(composite_options->watermark_geometry);
+  if (composite_options->compose_args != (char *) NULL)
+    composite_options->compose_args=(char *)
+      RelinquishMagickMemory(composite_options->compose_args);
 }
 
 WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
@@ -402,14 +392,14 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
 #define ThrowCompositeException(asperity,tag,option) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),asperity,tag,"`%s'", \
-     option == (char *) NULL ? strerror(errno) : option); \
+     option == (char *) NULL ? GetExceptionMessage(errno) : option); \
   DestroyComposite(); \
   return(MagickFalse); \
 }
 #define ThrowCompositeInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyComposite(); \
   return(MagickFalse); \
 }
@@ -463,12 +453,13 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
         {
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
-          (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
           return(MagickFalse);
         }
     }
   if (argc < 4)
-    CompositeUsage();
+    return(CompositeUsage());
   GetCompositeOptions(&composite_options);
   filename=(char *) NULL;
   format="%w,%h,%m";
@@ -488,7 +479,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
     ThrowCompositeException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   for (i=1; i < (long) (argc-1); i++)
   {
     option=argv[i];
@@ -517,7 +508,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
         /*
           Read input image.
         */
-        FireImageStack(MagickFalse,MagickTrue,pend);
+        FireImageStack(MagickFalse,MagickFalse,pend);
         filename=argv[i];
         if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           filename=argv[++i];
@@ -586,7 +577,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("blend",option+1) == 0)
           {
-            (void) CloneString(&composite_options.blend_geometry,(char *) NULL);
+            (void) CloneString(&composite_options.compose_args,(char *) NULL);
             if (*option == '+')
               break;
             i++;
@@ -594,10 +585,24 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowCompositeInvalidArgumentException(option,argv[i]);
-            (void) CloneString(&composite_options.blend_geometry,argv[i]);
+            (void) CloneString(&composite_options.compose_args,argv[i]);
             composite_options.compose=BlendCompositeOp;
             break;
           }
+        if (LocaleCompare("blur",option+1) == 0)
+          {
+            (void) CloneString(&composite_options.compose_args,(char *) NULL);
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompositeInvalidArgumentException(option,argv[i]);
+            (void) CloneString(&composite_options.compose_args,argv[i]);
+            composite_options.compose=BlurCompositeOp;
+            break;
+          }
         if (LocaleCompare("blue-primary",option+1) == 0)
           {
             if (*option == '+')
@@ -737,6 +742,8 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
                 "UnrecognizedImageCompression",argv[i]);
             break;
           }
+        if (LocaleCompare("concurrent",option+1) == 0)
+          break;
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
       }
       case 'd':
@@ -808,8 +815,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("displace",option+1) == 0)
           {
-            (void) CloneString(&composite_options.displace_geometry,
-              (char *) NULL);
+            (void) CloneString(&composite_options.compose_args,(char *) NULL);
             if (*option == '+')
               break;
             i++;
@@ -817,7 +823,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowCompositeInvalidArgumentException(option,argv[i]);
-            (void) CloneString(&composite_options.displace_geometry,argv[i]);
+            (void) CloneString(&composite_options.compose_args,argv[i]);
             composite_options.compose=DisplaceCompositeOp;
             break;
           }
@@ -848,8 +854,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("dissolve",option+1) == 0)
           {
-            (void) CloneString(&composite_options.dissolve_geometry,
-              (char *) NULL);
+            (void) CloneString(&composite_options.compose_args,(char *) NULL);
             if (*option == '+')
               break;
             i++;
@@ -857,11 +862,25 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowCompositeInvalidArgumentException(option,argv[i]);
-            (void) CloneString(&composite_options.dissolve_geometry,argv[i]);
+            (void) CloneString(&composite_options.compose_args,argv[i]);
             composite_options.compose=DissolveCompositeOp;
             break;
           }
-        if (LocaleCompare("method",option+1) == 0)
+        if (LocaleCompare("distort",option+1) == 0)
+          {
+            (void) CloneString(&composite_options.compose_args,(char *) NULL);
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompositeInvalidArgumentException(option,argv[i]);
+            (void) CloneString(&composite_options.compose_args,argv[i]);
+            composite_options.compose=DistortCompositeOp;
+            break;
+          }
+        if (LocaleCompare("dither",option+1) == 0)
           {
             long
               method;
@@ -877,6 +896,17 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("duration",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompositeInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
       }
       case 'e':
@@ -1019,7 +1049,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
       {
         if ((LocaleCompare("help",option+1) == 0) ||
             (LocaleCompare("-help",option+1) == 0))
-          CompositeUsage();
+          return(CompositeUsage());
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
       }
       case 'i':
@@ -1078,6 +1108,9 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -1094,11 +1127,30 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowCompositeException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowCompositeInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("list",option+1) == 0)
+          {
+            long
+              list;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            list=ParseMagickOption(MagickListOptions,MagickFalse,argv[i]);
+            if (list < 0)
+              ThrowCompositeException(OptionError,"UnrecognizedListType",
+                argv[i]);
+            (void) MogrifyImageInfo(image_info,(int) (i-j+1),(const char **)
+              argv+j,exception);
+            DestroyComposite();
+            return(MagickTrue);
+          }
         if (LocaleCompare("log",option+1) == 0)
           {
             if (*option == '+')
@@ -1230,7 +1282,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeInvalidArgumentException(option,argv[i]);
             break;
           }
-        if (LocaleCompare("respect-parenthesis",option+1) == 0)
+        if (LocaleNCompare("respect-parentheses",option+1,17) == 0)
           {
             respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
             break;
@@ -1333,13 +1385,26 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowCompositeInvalidArgumentException(option,argv[i]);
-            composite_options.stegano=atol(argv[i])+1;
+            composite_options.stegano=StringToLong(argv[i])+1;
             break;
           }
         if (LocaleCompare("stereo",option+1) == 0)
           {
-            composite_options.stereo=(*option == '-') ? MagickTrue :
-              MagickFalse;
+            MagickStatusType
+              flags;
+
+            composite_options.stereo=MagickFalse;
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompositeInvalidArgumentException(option,argv[i]);
+            flags=ParseAbsoluteGeometry(argv[i],&composite_options.offset);
+            if ((flags & YValue) == 0)
+              composite_options.offset.y=composite_options.offset.x;
+            composite_options.stereo=MagickTrue;
             break;
           }
         if (LocaleCompare("strip",option+1) == 0)
@@ -1378,7 +1443,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
         if (LocaleCompare("tile",option+1) == 0)
           {
             composite_options.tile=(*option == '-') ? MagickTrue : MagickFalse;
-            (void) CopyMagickString(argv[i]+1,"sans0",MaxTextExtent);
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             break;
           }
         if (LocaleCompare("transform",option+1) == 0)
@@ -1442,8 +1507,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("unsharp",option+1) == 0)
           {
-            (void) CloneString(&composite_options.unsharp_geometry,
-              (char *) NULL);
+            (void) CloneString(&composite_options.compose_args,(char *) NULL);
             if (*option == '+')
               break;
             i++;
@@ -1451,7 +1515,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowCompositeInvalidArgumentException(option,argv[i]);
-            (void) CloneString(&composite_options.unsharp_geometry,argv[i]);
+            (void) CloneString(&composite_options.compose_args,argv[i]);
             composite_options.compose=ThresholdCompositeOp;
             break;
           }
@@ -1466,7 +1530,8 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
           {
             (void) fprintf(stdout,"Version: %s\n",
               GetMagickVersion((unsigned long *) NULL));
-            (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
             break;
           }
         if (LocaleCompare("virtual-pixel",option+1) == 0)
@@ -1492,8 +1557,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("watermark",option+1) == 0)
           {
-            (void) CloneString(&composite_options.watermark_geometry,
-              (char *) NULL);
+            (void) CloneString(&composite_options.compose_args,(char *) NULL);
             if (*option == '+')
               break;
             i++;
@@ -1501,7 +1565,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowCompositeInvalidArgumentException(option,argv[i]);
-            (void) CloneString(&composite_options.watermark_geometry,argv[i]);
+            (void) CloneString(&composite_options.compose_args,argv[i]);
             composite_options.compose=ModulateCompositeOp;
             break;
           }
@@ -1539,10 +1603,11 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
     ThrowCompositeException(OptionError,"UnbalancedParenthesis",argv[i]);
   if (i-- != (long) (argc-1))
     ThrowCompositeException(OptionError,"MissingAnImageFilename",argv[i]);
-  if ((image == (Image *) NULL) ||
-      (GetImageListLength(image) < 2))
+  if ((image == (Image *) NULL) || (GetImageListLength(image) < 2))
+    ThrowCompositeException(OptionError,"MissingAnImageFilename",argv[argc-1]);
+  FinalizeImageSettings(image_info,image,MagickTrue);
+  if ((image == (Image *) NULL) || (GetImageListLength(image) < 2))
     ThrowCompositeException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  FinalizeImageSettings(image_info,image);
   /*
     Composite images.
   */
@@ -1553,13 +1618,14 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
   RemoveImageStack(mask_image);
   if (mask_image != (Image *) NULL)
     {
-      if (composite_options.compose == DisplaceCompositeOp)
+      if ((composite_options.compose == DisplaceCompositeOp) ||
+          (composite_options.compose == DistortCompositeOp))
         {
           /*
             Merge Y displacement into X displacement image.
           */
-          (void) CompositeImage(composite_image,CopyGreenCompositeOp,
-            mask_image,0,0);
+          (void) CompositeImage(composite_image,CopyGreenCompositeOp,mask_image,
+            0,0);
           mask_image=DestroyImage(mask_image);
         }
       else
@@ -1586,7 +1652,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
       text=InterpretImageProperties(image_info,images,format);
       if (text == (char *) NULL)
         ThrowCompositeException(ResourceLimitError,"MemoryAllocationFailed",
-          strerror(errno));
+          GetExceptionMessage(errno));
       (void) ConcatenateString(&(*metadata),text);
       (void) ConcatenateString(&(*metadata),"\n");
       text=DestroyString(text);
